// -----------------------------------
// Filename      : NetworkModeler.java
// Created       : 10/12/1999
// Last modified : 06/03/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// -----------------------------------

import mitrasim.exceptions.*;
import mitrasim.modeler.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class NetworkModeler
{
  private static String fLanguageFileName;

  public static void main(String[] argv)
  {
    // enable access to the resources for Mitrasim 2000
    Resources fResources = new Resources();
    fResources.loadConfiguration();

    try {
      parseCommandLine(argv);

      // access the language-file
      fResources.fMessages = new Messages(fLanguageFileName);
      System.out.println(fResources.fMessages.lookup("messageStartupBanner"));
      System.out.println("===> Network Modeler <===");
      System.out.println("");
      System.out.println(fResources.fMessages.lookup("messageThisLanguage"));
      System.out.println("");

      // find the amount of available memory
      Runtime runtime = Runtime.getRuntime ();
      runtime.gc (); // run garbage-collector
      long FreeMemory = runtime.freeMemory();
      long TotalMemory = runtime.totalMemory();
      System.out.println(fResources.fMessages.lookup("messageReportFreeMemory",String.valueOf(FreeMemory / 1024)));
      System.out.println(fResources.fMessages.lookup("messageReportTotalMemory",String.valueOf(TotalMemory / 1024)));
      System.out.println("");

      // launch the GUI
      System.out.println(fResources.fMessages.lookup("messageLaunchingGUI"));
      ModelerGUI modelerGUI = new ModelerGUI();
    }
    catch (GeneralError err) {
      switch (err.errorCategory()) {
        case GeneralError.kFileError : System.out.println("File error :");
                                       break;
        case GeneralError.kParseError : System.out.println("Parse error :");
                                        break;
        case GeneralError.kNetworkError : System.out.println("Network error :");
                                          break;
        case GeneralError.kCmdLineError : System.out.println("Commandline error :");
                                          break;
      }
      System.out.println(" => " + err.message());
    }
  }

  private static void parseCommandLine(String[] argv) throws GeneralError
  {
    // initialize the default filename
    fLanguageFileName = "language-english.data";

    if (argv.length > 0) {
      for (int i = 0; i < argv.length; ++i) {
        String fOption = argv[i];

        if (fOption.startsWith("--help")) {
          System.out.println("");
          System.out.println("Usage : NetworkModeler [options]");
          System.out.println("");
          System.out.println("Valid options are : --help");
          System.out.println("                    --language=Dutch|English");
          System.out.println("");
          System.out.println("If no language is specified, then English is assumed.");
          System.exit(0);
        }
        else if (fOption.startsWith("--language=")) {
          // cut the option, leaving the language
          fOption = fOption.substring(11);
          if (fOption.equalsIgnoreCase("English")) {
            fLanguageFileName = "language-english.data";
          }
          else {
            if (fOption.equalsIgnoreCase("Dutch")) {
              fLanguageFileName = "language-dutch.data";
            }
            else {
              throw (new GeneralError("Invalid language [" + fOption + "] specified.",
                                      GeneralError.kCmdLineError));
            }
          }
        }
        else {
          throw (new GeneralError("Unknown or incomplete option [" + fOption + "] specified.",
                                  GeneralError.kCmdLineError));
        }
      }
    }
  }
}
