// ------------------------------------
// Filename      : ConfiguratorGUI.java
// Created       : 21/01/2000
// Last modified : 22/06/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ------------------------------------

package mitrasim.configurator;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class ConfiguratorGUI extends Frame implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private int         fWinXSize;
  private int         fWinYSize;
  private Messages    fMessages;
  private final int   knvXSize              = 1;
  private final int   knvYSize              = 2;
  private final int   knvUpdateDelayMs      = 3;
  private final int   knvUpdateSteps        = 4;
  private final int   knvShowAboutBox       = 5;
  private final int   knmXSize              = 6;
  private final int   knmYSize              = 7;
  private final int   knmLegendXSize        = 8;
  private final int   knmLegendYSize        = 9;
  private final int   knmPixelsToMove       = 10;
  private final int   knmSelectionProximity = 11;
  private final int   knmShowAboutBox       = 12;
  private final int   krcXSize              = 13;
  private final int   krcYSize              = 14;
  private final int   krcShowAboutBox       = 15;
  private final int   kvisXSize             = 16;
  private final int   kvisYSize             = 17;
  private final int   kvisLegendXSize       = 18;
  private final int   kvisLegendYSize       = 19;
  private final int   kvisShowAboutBox      = 20;
  private CfgInputBox fCfgnvXSize;
  private CfgInputBox fCfgnvYSize;
  private CfgInputBox fCfgnvUpdateDelayMs;
  private CfgInputBox fCfgnvUpdateSteps;
  private CfgCheckBox fCfgnvShowAboutBox;
  private CfgInputBox fCfgnmXSize;
  private CfgInputBox fCfgnmYSize;
  private CfgInputBox fCfgnmLegendXSize;
  private CfgInputBox fCfgnmLegendYSize;
  private CfgInputBox fCfgnmPixelsToMove;
  private CfgInputBox fCfgnmSelectionProximity;
  private CfgCheckBox fCfgnmShowAboutBox;
  private CfgInputBox fCfgrcXSize;
  private CfgInputBox fCfgrcYSize;
  private CfgCheckBox fCfgrcShowAboutBox;
  private CfgInputBox fCfgvisXSize;
  private CfgInputBox fCfgvisYSize;
  private CfgInputBox fCfgvisLegendXSize;
  private CfgInputBox fCfgvisLegendYSize;
  private CfgCheckBox fCfgvisShowAboutBox;
  private boolean     fChanged;

  public ConfiguratorGUI()
  {
    super();

    fResources = new Resources();
    fWinXSize = fResources.fcfgXSize;
    fWinYSize = fResources.fcfgYSize;
    fMessages = fResources.fMessages;
    fChanged = false;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setSize(fWinXSize,fWinYSize);
    setResizable(false);
    setLocation(0,0);
    setTitle(fMessages.lookup("textConfiguratorWindowTitle"));
    setBackground(Color.lightGray);
    setForeground(Color.black);
    setIconImage(Toolkit.getDefaultToolkit().getImage("M2000Icon.jpg"));

    setLayout(new BorderLayout());

    // create panels containing settings- and control-commands
    Panel SettingsPanel = new Panel();
    SettingsPanel.setLayout(new GridLayout(14,4));
    Panel UpperControlPanel = new Panel();
    UpperControlPanel.setLayout(new GridLayout(1,3));
    Panel LowerControlPanel = new Panel();
    LowerControlPanel.setLayout(new GridLayout(2,2));
    Panel ControlPanel = new Panel();
    ControlPanel.setLayout(new BorderLayout());

    SettingsPanel.add(new Label());
    Label label = new Label("Network Modeler",Label.CENTER);
    label.setFont(new Font(fMessages.lookup("textFontName"),Font.BOLD,18));
    SettingsPanel.add(label);

    SettingsPanel.add(new Label());
    label = new Label("Network Viewer",Label.CENTER);
    label.setFont(new Font(fMessages.lookup("textFontName"),Font.BOLD,18));
    SettingsPanel.add(label);

    SettingsPanel.add(new Label(fMessages.lookup("textnmXSize") + " : ",Label.RIGHT));
    fCfgnmXSize = new CfgInputBox(knmXSize);
    SettingsPanel.add(fCfgnmXSize);

    SettingsPanel.add(new Label(fMessages.lookup("textnvXSize") + " : ",Label.RIGHT));
    fCfgnvXSize = new CfgInputBox(knvXSize);
    SettingsPanel.add(fCfgnvXSize);

    SettingsPanel.add(new Label(fMessages.lookup("textnmYSize") + " : ",Label.RIGHT));
    fCfgnmYSize = new CfgInputBox(knmYSize);
    SettingsPanel.add(fCfgnmYSize);

    SettingsPanel.add(new Label(fMessages.lookup("textnvYSize") + " : ",Label.RIGHT));
    fCfgnvYSize = new CfgInputBox(knvYSize);
    SettingsPanel.add(fCfgnvYSize);

    SettingsPanel.add(new Label(fMessages.lookup("textnmLegendXSize") + " : ",Label.RIGHT));
    fCfgnmLegendXSize = new CfgInputBox(knmLegendXSize);
    SettingsPanel.add(fCfgnmLegendXSize);

    SettingsPanel.add(new Label(fMessages.lookup("textnvUpdateDelayMs") + " : ",Label.RIGHT));
    fCfgnvUpdateDelayMs = new CfgInputBox(knvUpdateDelayMs);
    SettingsPanel.add(fCfgnvUpdateDelayMs);

    SettingsPanel.add(new Label(fMessages.lookup("textnmLegendYSize") + " : ",Label.RIGHT));
    fCfgnmLegendYSize = new CfgInputBox(knmLegendYSize);
    SettingsPanel.add(fCfgnmLegendYSize);

    SettingsPanel.add(new Label(fMessages.lookup("textnvUpdateSteps") + " : ",Label.RIGHT));
    fCfgnvUpdateSteps = new CfgInputBox(knvUpdateSteps);
    SettingsPanel.add(fCfgnvUpdateSteps);

    SettingsPanel.add(new Label(fMessages.lookup("textnmPixelsToMove") + " : ",Label.RIGHT));
    fCfgnmPixelsToMove = new CfgInputBox(knmPixelsToMove);
    SettingsPanel.add(fCfgnmPixelsToMove);

    SettingsPanel.add(new Label(fMessages.lookup("textnvShowAboutBox") + " : ",Label.RIGHT));
    fCfgnvShowAboutBox = new CfgCheckBox(knvShowAboutBox);
    SettingsPanel.add(fCfgnvShowAboutBox);

    SettingsPanel.add(new Label(fMessages.lookup("textnmSelectionProximity") + " : ",Label.RIGHT));
    fCfgnmSelectionProximity = new CfgInputBox(knmSelectionProximity);
    SettingsPanel.add(fCfgnmSelectionProximity);

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textnmShowAboutBox") + " : ",Label.RIGHT));
    fCfgnmShowAboutBox = new CfgCheckBox(knmShowAboutBox);
    SettingsPanel.add(fCfgnmShowAboutBox);

    SettingsPanel.add(new Label());
    label = new Label("Visualizer",Label.CENTER);
    label.setFont(new Font(fMessages.lookup("textFontName"),Font.BOLD,18));
    SettingsPanel.add(label);

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textvisXSize") + " : ",Label.RIGHT));
    fCfgvisXSize = new CfgInputBox(kvisXSize);
    SettingsPanel.add(fCfgvisXSize);

    SettingsPanel.add(new Label());
    label = new Label("Remote Control",Label.CENTER);
    label.setFont(new Font(fMessages.lookup("textFontName"),Font.BOLD,18));
    SettingsPanel.add(label);

    SettingsPanel.add(new Label(fMessages.lookup("textvisYSize") + " : ",Label.RIGHT));
    fCfgvisYSize = new CfgInputBox(kvisYSize);
    SettingsPanel.add(fCfgvisYSize);

    SettingsPanel.add(new Label(fMessages.lookup("textrcXSize") + " : ",Label.RIGHT));
    fCfgrcXSize = new CfgInputBox(krcXSize);
    SettingsPanel.add(fCfgrcXSize);

    SettingsPanel.add(new Label(fMessages.lookup("textvisLegendXSize") + " : ",Label.RIGHT));
    fCfgvisLegendXSize = new CfgInputBox(kvisLegendXSize);
    SettingsPanel.add(fCfgvisLegendXSize);

    SettingsPanel.add(new Label(fMessages.lookup("textrcYSize") + " : ",Label.RIGHT));
    fCfgrcYSize = new CfgInputBox(krcYSize);
    SettingsPanel.add(fCfgrcYSize);

    SettingsPanel.add(new Label(fMessages.lookup("textvisLegendYSize") + " : ",Label.RIGHT));
    fCfgvisLegendYSize = new CfgInputBox(kvisLegendYSize);
    SettingsPanel.add(fCfgvisLegendYSize);

    SettingsPanel.add(new Label(fMessages.lookup("textrcShowAboutBox") + " : ",Label.RIGHT));
    fCfgrcShowAboutBox = new CfgCheckBox(krcShowAboutBox);
    SettingsPanel.add(fCfgrcShowAboutBox);

    SettingsPanel.add(new Label(fMessages.lookup("textvisShowAboutBox") + " : ",Label.RIGHT));
    fCfgvisShowAboutBox = new CfgCheckBox(kvisShowAboutBox);
    SettingsPanel.add(fCfgvisShowAboutBox);

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    Button fButton;

    fButton = new Button(fMessages.lookup("button800x600"));
    fButton.addActionListener(this);
    UpperControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("button1024x768"));
    fButton.addActionListener(this);
    UpperControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("button1280x1024"));
    fButton.addActionListener(this);
    UpperControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonRestoreDefaults"));
    fButton.addActionListener(this);
    LowerControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonAbout"));
    fButton.addActionListener(this);
    LowerControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonSaveSettings"));
    fButton.addActionListener(this);
    LowerControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonQuit"));
    fButton.setBackground(Color.red);
    fButton.addActionListener(this);
    LowerControlPanel.add(fButton);

    ControlPanel.add("North",UpperControlPanel);
    ControlPanel.add("South",LowerControlPanel);

    // add the controlpanel to the window
    add("North",SettingsPanel);
    add("South",ControlPanel);

    // add the windowlistener
    addWindowListener(this);

    pack();
    show();

    // show the about-box
    AboutBox about = new AboutBox(this,"Configurator");
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    if (fChanged) {
      ConfirmationBox Confirmation = new ConfirmationBox(this,
                                                         fMessages.lookup("messageSaveSettings"),
                                                         fMessages.lookup("buttonYes"),
                                                         fMessages.lookup("buttonNo"));
      if (Confirmation.confirmed()) {
        saveSettings();
      }
    }
    // quit the running program
    System.exit(0);
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("button800x600"))) {
      fResources.fnvXSize = fResources.knvDefault800x600XSize;
      fResources.fnvYSize = fResources.knvDefault800x600YSize;
      fResources.fnmXSize = fResources.knmDefault800x600XSize;
      fResources.fnmYSize = fResources.knmDefault800x600YSize;
      fResources.frcXSize = fResources.krcDefault800x600XSize;
      fResources.frcYSize = fResources.krcDefault800x600YSize;
      fResources.fvisXSize = fResources.kvisDefault800x600XSize;
      fResources.fvisYSize = fResources.kvisDefault800x600YSize;
      fChanged = true;
      updateAll();
    }

    if (Command.equals(fMessages.lookup("button1024x768"))) {
      fResources.fnvXSize = fResources.knvDefault1024x768XSize;
      fResources.fnvYSize = fResources.knvDefault1024x768YSize;
      fResources.fnmXSize = fResources.knmDefault1024x768XSize;
      fResources.fnmYSize = fResources.knmDefault1024x768YSize;
      fResources.frcXSize = fResources.krcDefault1024x768XSize;
      fResources.frcYSize = fResources.krcDefault1024x768YSize;
      fResources.fvisXSize = fResources.kvisDefault1024x768XSize;
      fResources.fvisYSize = fResources.kvisDefault1024x768YSize;
      fChanged = true;
      updateAll();
    }

    if (Command.equals(fMessages.lookup("button1280x1024"))) {
      fResources.fnvXSize = fResources.knvDefault1280x1024XSize;
      fResources.fnvYSize = fResources.knvDefault1280x1024YSize;
      fResources.fnmXSize = fResources.knmDefault1280x1024XSize;
      fResources.fnmYSize = fResources.knmDefault1280x1024YSize;
      fResources.frcXSize = fResources.krcDefault1280x1024XSize;
      fResources.frcYSize = fResources.krcDefault1280x1024YSize;
      fResources.fvisXSize = fResources.kvisDefault1280x1024XSize;
      fResources.fvisYSize = fResources.kvisDefault1280x1024YSize;
      fChanged = true;
      updateAll();
    }

    if (Command.equals(fMessages.lookup("buttonRestoreDefaults"))) {
      fResources.fnvXSize = fResources.knvDefault800x600XSize;
      fResources.fnvYSize = fResources.knvDefault800x600YSize;
      fResources.fnvUpdateDelayMs = fResources.knvDefaultUpdateDelayMs;
      fResources.fnvUpdateSteps = fResources.knvDefaultUpdateSteps;
      fResources.fnvShowAboutBox = fResources.knvDefaultShowAboutBox;
      fResources.fnmXSize = fResources.knmDefault800x600XSize;
      fResources.fnmYSize = fResources.knmDefault800x600YSize;
      fResources.fnmLegendXSize = fResources.knmDefaultLegendXSize;
      fResources.fnmLegendYSize = fResources.knmDefaultLegendYSize;
      fResources.fnmPixelsToMove = fResources.knmDefaultPixelsToMove;
      fResources.fnmSelectionProximity = fResources.knmDefaultSelectionProximity;
      fResources.fnmShowAboutBox = fResources.knmDefaultShowAboutBox;
      fResources.frcXSize = fResources.krcDefault800x600XSize;
      fResources.frcYSize = fResources.krcDefault800x600YSize;
      fResources.frcShowAboutBox = fResources.krcDefaultShowAboutBox;
      fResources.fvisXSize = fResources.kvisDefault800x600XSize;
      fResources.fvisYSize = fResources.kvisDefault800x600YSize;
      fResources.fvisLegendXSize = fResources.kvisDefaultLegendXSize;
      fResources.fvisLegendYSize = fResources.kvisDefaultLegendYSize;
      fResources.fvisShowAboutBox = fResources.kvisDefaultShowAboutBox;
      fChanged = true;
      updateAll();
    }

    if (Command.equals(fMessages.lookup("buttonAbout"))) {
      AboutBox fAbout = new AboutBox(this,"Configurator");
    }

    if (Command.equals(fMessages.lookup("buttonSaveSettings"))) {
      saveSettings();
      fChanged = false;
    }

    if (Command.equals(fMessages.lookup("buttonQuit"))) {
      windowClosing(null);
    }
  }

  private void updateAll()
  {
    // the network-viewer
    fCfgnvXSize.update();
    fCfgnvYSize.update();
    fCfgnvUpdateDelayMs.update();
    fCfgnvUpdateSteps.update();
    fCfgnvShowAboutBox.update();

    // the network-modeler
    fCfgnmXSize.update();
    fCfgnmYSize.update();
    fCfgnmLegendXSize.update();
    fCfgnmLegendYSize.update();
    fCfgnmPixelsToMove.update();
    fCfgnmSelectionProximity.update();
    fCfgnmShowAboutBox.update();

    // the remote control
    fCfgrcXSize.update();
    fCfgrcYSize.update();
    fCfgrcShowAboutBox.update();

    // the visualizer
    fCfgvisXSize.update();
    fCfgvisYSize.update();
    fCfgvisLegendXSize.update();
    fCfgvisLegendYSize.update();
    fCfgvisShowAboutBox.update();
  }

  private void saveSettings()
  {
    try {
      fResources.saveConfiguration();

      MessageBox messageBox = new MessageBox(this,
                                             fMessages.lookup("messageSettingsSaved"),
                                             MessageBox.kMessage); 
    }
    catch (IOException e) {
      MessageBox ErrorBox = new MessageBox(this,
                                           fMessages.lookup("errorSettingsNotSaved"),
                                           MessageBox.kError); 
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == knvXSize) {
        setText(String.valueOf(fResources.fnvXSize));
      }
      else if (fInputBox == knvYSize) {
        setText(String.valueOf(fResources.fnvYSize));
      }
      else if (fInputBox == knvUpdateDelayMs) {
        setText(String.valueOf(fResources.fnvUpdateDelayMs));
      }
      else if (fInputBox == knvUpdateSteps) {
        setText(String.valueOf(fResources.fnvUpdateSteps));
      }
      else if (fInputBox == knmXSize) {
        setText(String.valueOf(fResources.fnmXSize));
      }
      else if (fInputBox == knmYSize) {
        setText(String.valueOf(fResources.fnmYSize));
      }
      else if (fInputBox == knmLegendXSize) {
        setText(String.valueOf(fResources.fnmLegendXSize));
      }
      else if (fInputBox == knmLegendYSize) {
        setText(String.valueOf(fResources.fnmLegendYSize));
      }
      else if (fInputBox == knmPixelsToMove) {
        setText(String.valueOf(fResources.fnmPixelsToMove));
      }
      else if (fInputBox == knmSelectionProximity) {
        setText(String.valueOf(fResources.fnmSelectionProximity));
      }
      else if (fInputBox == krcXSize) {
        setText(String.valueOf(fResources.frcXSize));
      }
      else if (fInputBox == krcYSize) {
        setText(String.valueOf(fResources.frcYSize));
      }
      else if (fInputBox == kvisXSize) {
        setText(String.valueOf(fResources.fvisXSize));
      }
      else if (fInputBox == kvisYSize) {
        setText(String.valueOf(fResources.fvisYSize));
      }
      else if (fInputBox == kvisLegendXSize) {
        setText(String.valueOf(fResources.fvisLegendXSize));
      }
      else if (fInputBox == kvisLegendYSize) {
        setText(String.valueOf(fResources.fvisLegendYSize));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int NewInput = 0;
      try {
        NewInput = (new Integer(e.getActionCommand())).intValue();
      }
      catch (NumberFormatException exc) {
        if (fInputBox == knvXSize) {
          NewInput = fResources.fnvXSize;
        }
        else if (fInputBox == knvYSize) {
          NewInput = fResources.fnvYSize;
        }
        else if (fInputBox == knvUpdateDelayMs) {
          NewInput = fResources.fnvUpdateDelayMs;
        }
        else if (fInputBox == knvUpdateSteps) {
          NewInput = fResources.fnvUpdateSteps;
        }
        else if (fInputBox == knmXSize) {
          NewInput = fResources.fnmXSize;
        }
        else if (fInputBox == knmYSize) {
          NewInput = fResources.fnmYSize;
        }
        else if (fInputBox == knmLegendXSize) {
          NewInput = fResources.fnmLegendXSize;
        }
        else if (fInputBox == knmLegendYSize) {
          NewInput = fResources.fnmLegendYSize;
        }
        else if (fInputBox == knmPixelsToMove) {
          NewInput = fResources.fnmPixelsToMove;
        }
        else if (fInputBox == knmSelectionProximity) {
          NewInput = fResources.fnmSelectionProximity;
        }
        else if (fInputBox == krcXSize) {
          NewInput = fResources.frcXSize;
        }
        else if (fInputBox == krcYSize) {
          NewInput = fResources.frcYSize;
        }
        else if (fInputBox == kvisXSize) {
          NewInput = fResources.fvisXSize;
        }
        else if (fInputBox == kvisYSize) {
          NewInput = fResources.fvisYSize;
        }
        else if (fInputBox == kvisLegendXSize) {
          NewInput = fResources.fvisLegendXSize;
        }
        else if (fInputBox == kvisLegendYSize) {
          NewInput = fResources.fvisLegendYSize;
        }
      }

      if (fInputBox == knvXSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnvXSize = NewInput;
      }
      else if (fInputBox == knvYSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnvYSize = NewInput;
      }
      else if (fInputBox == knvUpdateDelayMs) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnvUpdateDelayMs = NewInput;
      }
      else if (fInputBox == knvUpdateSteps) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnvUpdateSteps = NewInput;
      }
      else if (fInputBox == knmXSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnmXSize = NewInput;
      }
      else if (fInputBox == knmYSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnmYSize = NewInput;
      }
      else if (fInputBox == knmLegendXSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnmLegendXSize = NewInput;
      }
      else if (fInputBox == knmLegendYSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnmLegendYSize = NewInput;
      }
      else if (fInputBox == knmPixelsToMove) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnmPixelsToMove = NewInput;
      }
      else if (fInputBox == knmSelectionProximity) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fnmSelectionProximity = NewInput;
      }
      else if (fInputBox == krcXSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.frcXSize = NewInput;
      }
      else if (fInputBox == krcYSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.frcYSize = NewInput;
      }
      else if (fInputBox == kvisXSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fvisXSize = NewInput;
      }
      else if (fInputBox == kvisYSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fvisYSize = NewInput;
      }
      else if (fInputBox == kvisLegendXSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fvisLegendXSize = NewInput;
      }
      else if (fInputBox == kvisLegendYSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        fResources.fvisLegendYSize = NewInput;
      }
      fChanged = true;

      update();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgCheckBox extends Checkbox implements ItemListener
  {
    private int fCheckBox;

    public CfgCheckBox(int checkBox)
    {
      super();
      fCheckBox = checkBox;
      addItemListener(this);
      update();
    }

    synchronized public void update()
    {
      if (fCheckBox == knvShowAboutBox) {
        setState(fResources.fnvShowAboutBox);
        if (fResources.fnvShowAboutBox) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == knmShowAboutBox) {
        setState(fResources.fnmShowAboutBox);
        if (fResources.fnmShowAboutBox) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == krcShowAboutBox) {
        setState(fResources.frcShowAboutBox);
        if (fResources.frcShowAboutBox) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == kvisShowAboutBox) {
        setState(fResources.fvisShowAboutBox);
        if (fResources.fvisShowAboutBox) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fCheckBox == knvShowAboutBox) {
        fResources.fnvShowAboutBox = !fResources.fnvShowAboutBox;
        fChanged = true;
      }
      else if (fCheckBox == knmShowAboutBox) {
        fResources.fnmShowAboutBox = !fResources.fnmShowAboutBox;
        fChanged = true;
      }
      else if (fCheckBox == krcShowAboutBox) {
        fResources.frcShowAboutBox = !fResources.frcShowAboutBox;
        fChanged = true;
      }
      else if (fCheckBox == kvisShowAboutBox) {
        fResources.fvisShowAboutBox = !fResources.fvisShowAboutBox;
        fChanged = true;
      }

      update();
    }
  }
}
