// ----------------------------------
// Filename      : WVMapping.java
// Created       : 02/11/1999
// Last modified : 02/11/1999
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.engine3d;

import mitrasim.engine3d.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class WVMapping
{
  // the coordinates of the world-window
  private float fWorldX1;
  private float fWorldY1;
  private float fWorldX2;
  private float fWorldY2;

  // the coordinates of the viewport-window
  private int fViewportX1;
  private int fViewportY1;
  private int fViewportX2;
  private int fViewportY2;

  // the window-to-viewport-scaling factors
  private float fScaleX;
  private float fScaleY;

  public WVMapping(float wX1, float wY1, float wX2, float wY2,
                   int vX1, int vY1, int vX2, int vY2)
  {
    setWVMapping(wX1,wY1,wX2,wY2,vX1,vY1,vX2,vY2);
  }

  public void setWVMapping(float wX1, float wY1, float wX2, float wY2,
                           int vX1, int vY1, int vX2, int vY2)
  {
    fWorldX1 = wX1;
    fWorldY1 = wY1;
    fWorldX2 = wX2;
    fWorldY2 = wY2;

    fViewportX1 = vX1;
    fViewportY1 = vY1;
    fViewportX2 = vX2;
    fViewportY2 = vY2;

    fScaleX = (fViewportX2 - fViewportX1) / (fWorldX2 - fWorldX1);
    fScaleY = (fViewportY2 - fViewportY1) / (fWorldY2 - fWorldY1);

    // make the scaling uniform
    if (fScaleX < fScaleY) {
      fScaleY = fScaleX;
    }
    else {
      fScaleX = fScaleY;
    }
  }

  public ViewportPoint map(Point2D wp)
  {
    ViewportPoint vp = new ViewportPoint();

    vp.fX = fViewportX1 + Math.round((wp.fX - fWorldX1) * fScaleX);
    vp.fY = fViewportY1 + Math.round((wp.fY - fWorldY1) * fScaleY);

    return vp;
  }
}
