// ---------------------------------------
// Filename      : GeneralSettingsBox.java
// Created       : 09/02/2000
// Last modified : 27/04/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ---------------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import mitrasim.network.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class GeneralSettingsBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  public  float       fLaneWidth;
  public  float       fRailroadWidth;
  public  String      fTitle;
  private final int   kLaneWidth     = 1;
  private final int   kRailroadWidth = 2;
  private final int   kTitle         = 3;
  private CfgInputBox fCfgLaneWidth;
  private CfgInputBox fCfgRailroadWidth;
  private CfgInputBox fCfgTitle;
  private boolean     fCanceled;

  public GeneralSettingsBox(Frame parent, float LaneWidth, float RailroadWidth, String Title)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fLaneWidth = LaneWidth;
    fRailroadWidth = RailroadWidth;
    fTitle = Title;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textGeneralSettingsDialogTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // define the controlpanels
    Panel SettingsPanel = new Panel();
    Panel ControlPanel = new Panel();
    SettingsPanel.setLayout(new GridLayout(4,3));
    ControlPanel.setLayout(new GridLayout(1,2));

    SettingsPanel.add(new Label(fMessages.lookup("textNetworkLaneWidth") + " = ",Label.RIGHT));
    fCfgLaneWidth = new CfgInputBox(kLaneWidth);
    SettingsPanel.add(fCfgLaneWidth);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textNetworkRailroadWidth") + " = ",Label.RIGHT));
    fCfgRailroadWidth = new CfgInputBox(kRailroadWidth);
    SettingsPanel.add(fCfgRailroadWidth);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textNetworkTitle") + " = ",Label.RIGHT));
    fCfgTitle = new CfgInputBox(kTitle);
    SettingsPanel.add(fCfgTitle);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    button = new Button(fMessages.lookup("buttonOk"));
    button.addActionListener(this);
    ControlPanel.add(button);

    // add the controlpanel
    setLayout(new BorderLayout());
    add(SettingsPanel,"North");
    add(ControlPanel,"South");

    // add the windowlistener
    addWindowListener(this);

    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  private void update()
  {
    fCfgLaneWidth.update();
    fCfgTitle.update();
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kLaneWidth) {
        setText(String.valueOf(fLaneWidth));
      }
      else if (fInputBox == kRailroadWidth) {
        setText(String.valueOf(fRailroadWidth));
      }
      else if (fInputBox == kTitle) {
        setText(fTitle);
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      float FloatInput = 0;
      String StringInput = "";
      try {
        if (fInputBox != kTitle) {
          FloatInput = (new Float(e.getActionCommand())).floatValue();
        }
        else {
          StringInput = e.getActionCommand();
        }
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kLaneWidth) {
          FloatInput = fLaneWidth;
        }
        else if (fInputBox == kRailroadWidth) {
          FloatInput = fRailroadWidth;
        }
      }

      if (fInputBox == kLaneWidth) {
        if (FloatInput <= 0.0f) {
          FloatInput = 1.0f;
        }
        fLaneWidth = FloatInput;        
      }
      else if (fInputBox == kRailroadWidth) {
        if (FloatInput <= 0.0f) {
          FloatInput = 1.0f;
        }
        fRailroadWidth = FloatInput;        
      }
      else if (fInputBox == kTitle) {
        fTitle = StringInput;        
      }

      update();
    }
  }
}
