// -----------------------------------
// Filename      : LaneChangesBox.java
// Created       : 05/03/2000
// Last modified : 06/03/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// -----------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.network.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class LaneChangesBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources     fResources;
  private Messages      fMessages;
  private Link          fLink;
  private CfgCheckBox[] fCfgLaneChanges;
  private LinkCanvas    fLinkCanvas;
  private boolean       fCanceled;
  private boolean       fLinkCanvasDefined;

  public LaneChangesBox(Frame parent, Link link)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fLink = link;
    fCanceled = true;
    fLinkCanvasDefined = false;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textLaneChangesBoxTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // define the controlpanels
    Panel LaneChangesPanel = new Panel();
    Panel LinkTypePanel = new Panel();
    Panel ControlPanel = new Panel();
    LaneChangesPanel.setLayout(new GridLayout(fLink.fNrOfLaneChangePermissions / 2,4));
    LinkTypePanel.setLayout(new GridLayout(1,1));
    ControlPanel.setLayout(new GridLayout(1,2));

    fCfgLaneChanges = new CfgCheckBox[fLink.fNrOfLaneChangePermissions];

    for (int i = 0; i < fLink.fNrOfLaneChangePermissions; ++i) {
      String LaneChange = "";
      if ((i % 2) == 0) {
        LaneChange = String.valueOf((i / 2) + 1) +
                                    "->" +
                                    String.valueOf((i / 2) + 2);
      }
      else {
        LaneChange = String.valueOf((i / 2) + 2) +
                                    "->" +
                                    String.valueOf((i / 2) + 1);
      }
      LaneChangesPanel.add(new Label(LaneChange + " ",Label.RIGHT));
      fCfgLaneChanges[i] = new CfgCheckBox(i);
      LaneChangesPanel.add(fCfgLaneChanges[i]);
    }

    String LinkType = fLink.translate(fMessages);
    if ((fLink.fType == Link.kSeparatedOnRamp) ||
        (fLink.fType == Link.kMergedOnRamp) ||
        (fLink.fType == Link.kSeparatedOffRamp) ||
        (fLink.fType == Link.kMergedOffRamp) ||
        (fLink.fType == Link.kNarrowing) ||
        (fLink.fType == Link.kWidening)) {
      if (fLink.fLocation == Link.kLeft) {
        LinkType = fMessages.lookup("textLeftLocation") + " " + LinkType;
      }
      else {
        LinkType = fMessages.lookup("textRightLocation") + " " + LinkType;
      }
    }
    LinkTypePanel.add(new Label("(" + LinkType + ")",Label.CENTER));

    fLinkCanvas = new LinkCanvas(fResources.knmLinkCanvasXSize,fResources.knmLinkCanvasLaneHeight);
    fLinkCanvasDefined = true;

    Button ControlButton;
    ControlButton = new Button(fMessages.lookup("buttonCancel"));
    ControlButton.addActionListener(this);
    ControlPanel.add(ControlButton);

    ControlButton = new Button(fMessages.lookup("buttonOk"));
    ControlButton.addActionListener(this);
    ControlPanel.add(ControlButton);

    setLayout(new BorderLayout());
    Panel UpperPanel = new Panel();
    UpperPanel.setLayout(new BorderLayout());
    UpperPanel.add("North",LaneChangesPanel);
    UpperPanel.add("South",LinkTypePanel);
    add("North",UpperPanel);
    add("Center",fLinkCanvas);
    add("South",ControlPanel);

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public Link getLink()
  {
    return fLink;
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgCheckBox extends Checkbox implements ItemListener
  {
    private int fIndex;

    public CfgCheckBox(int index)
    {
      super();
      fIndex = index;
      addItemListener(this);
      update();
    }

    synchronized public void update()
    {
      if (fLink.fLaneChangePermissions[fIndex] == Link.kAllowed) {
        setState(true);
        setForeground(Color.black);
        setLabel(" (" + fMessages.lookup("textAllowed") + ")");
      }
      else if (fLink.fLaneChangePermissions[fIndex] == Link.kNotAllowed) {
        setState(false);
        setForeground(Color.blue);
        setLabel(" (" + fMessages.lookup("textNotAllowed") + ")");
      }

      if (fLinkCanvasDefined) {
        fLinkCanvas.repaint();
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fLink.fLaneChangePermissions[fIndex] == Link.kAllowed) {
        fLink.fLaneChangePermissions[fIndex] = Link.kNotAllowed;
      }
      else if (fLink.fLaneChangePermissions[fIndex] == Link.kNotAllowed) {
        fLink.fLaneChangePermissions[fIndex] = Link.kAllowed;
      }
      update();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  // the link-canvas
  private class LinkCanvas extends Canvas
  {
    // the canvas-draw-area
    private int cx1;
    private int cy1;
    private int cx2;
    private int cy2;
    private int cwidth;
    private int cheight;

    public LinkCanvas(int width, int laneheight)
    {
      super();

      setSize(width,laneheight * ((fLink.fNrOfLaneChangePermissions / 2) + 1));
    }

    public void update(Graphics g)
    {
      paint(g);
    }

    public void paint(Graphics g)
    {
      // define the boundaries for drawing
      cx1 = 5;
      cy1 = 5;
      cx2 = (cx1 + getSize().width) - (getInsets().right + 5 + 5);
      cy2 = (cy1 + getSize().height) - (getInsets().bottom + 5 + 5);
      cwidth = cx2 - cx1;
      cheight = cy2 - cy1;

      // activate clipping of graphics primitives
      g.setClip(cx1,cy1,cwidth,cheight);

      g.setColor(Color.black);
      g.fillRect(cx1,cy1,cwidth,cheight);

      g.setColor(Color.white);
      g.drawRect(cx1,cy1,cwidth - 1,cheight - 1);

      g.setColor(Color.white);
      int dy = cheight / ((fLink.fNrOfLaneChangePermissions / 2) + 1);
      for (int i = 0; i < fLink.fNrOfLaneChangePermissions; ++i) {
        int yPos = cy2 - (dy * ((i / 2) + 1));

        if ((fLink.fLaneChangePermissions[i] == Link.kAllowed) &&
            (fLink.fLaneChangePermissions[i + 1] == Link.kAllowed)) {
          drawDashedLine(g,yPos);
        }
        else if ((fLink.fLaneChangePermissions[i] == Link.kNotAllowed) &&
                 (fLink.fLaneChangePermissions[i + 1] == Link.kNotAllowed)) {
          drawContinuousLine(g,yPos);
        }
        else {
          if (fLink.fLaneChangePermissions[i] == Link.kAllowed) {
            drawDashedLine(g,yPos + 2);
          }
          else if (fLink.fLaneChangePermissions[i] == Link.kNotAllowed) {
            drawContinuousLine(g,yPos + 2);
          }

          if (fLink.fLaneChangePermissions[i + 1] == Link.kAllowed) {
            drawDashedLine(g,yPos - 3);
          }
          else if (fLink.fLaneChangePermissions[i + 1] == Link.kNotAllowed) {
            drawContinuousLine(g,yPos - 3);
          }
        }

        // advance to the next couple lanechanges
        ++i;
      }

      g.setColor(Color.yellow);
      for (int i = 0; i <= fLink.fNrOfLaneChangePermissions; ++i) {
        int yPos = cy2 - (dy * ((i / 2) + 1));
        g.drawString(String.valueOf((i / 2) + 1),cx1 + 5,yPos + 10);
      }
    }

    private void drawDashedLine(Graphics g,int yPos)
    {
      final int NrOfDashes = 21;
      int dx = (cwidth - 10) / (NrOfDashes + 1);
      for (int i = 0; i <= (NrOfDashes + 1); ++i) {
        int xPos = (cx1 + 21) + (dx * i);
        g.drawLine(xPos,yPos,xPos + dx,yPos);
        g.drawLine(xPos,yPos + 1,xPos + dx,yPos + 1);
        // skip to the next dash
        ++i;
      }
    }

    private void drawContinuousLine(Graphics g,int yPos)
    {
      g.drawLine(cx1 + 21,yPos,cx2 - 1,yPos);
      g.drawLine(cx1 + 21,yPos + 1,cx2 - 1,yPos + 1);
    }
  }
}
