// ----------------------------------
// Filename      : LinkBox.java
// Created       : 31/01/2000
// Last modified : 21/04/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import mitrasim.network.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class LinkBox extends Dialog implements WindowListener, ActionListener
{
  // the different dialog-modes
  public static final int kAddMode  = 1;
  public static final int kEditMode = 2;

  // internal datastructures
  private Resources        fResources;
  private Messages         fMessages;
  private int              fDialogMode;
  private Link             fLink;
  private final int        kID                      = 1;
  private final int        kNrOfLanes               = 2;
  private final int        kNodes                   = 3;
  private final int        kSpeedLimit              = 4;
  private final int        kIsTerminator            = 5;
  private final int        kMainLinkConnectedToLink = 6;
  private final int        kMainLinkConnectedToPart = 7;
  private final int        kType                    = 8;
  private final int        kLocation                = 9;
  private final int        kOffRampConnectedToLink  = 10;
  private final int        kOffRampConnectedToPart  = 11;
  private final int        kNrOfLanesOnRamp         = 12;
  private final int        kNewNrOfLanes            = 13;
  private final int        kName                    = 14;
  private Label            fLabelID;
  private CfgInputBox      fCfgID;
  private CfgItemList      fCfgNrOfLanes;
  private CfgNodesInputBox fCfgNodes;
  private Label            fLabelNodes;
  private CfgScrollBar     fCfgSpeedLimit;
  private Label            fLabelSpeedLimit;
  private Label            fLabelIsTerminator;
  private CfgCheckBox      fCfgIsTerminator;
  private Label            fLabelMainLinkConnectedToLink;
  private CfgInputBox      fCfgMainLinkConnectedToLink;
  private Label            fLabelMainLinkConnectedToPart;
  private CfgItemList      fCfgMainLinkConnectedToPart;
  private CfgItemList      fCfgType;
  private Label            fLabelLocation;
  private CfgItemList      fCfgLocation;
  private Label            fLabelOffRampConnectedToLink;
  private CfgInputBox      fCfgOffRampConnectedToLink;
  private Label            fLabelOffRampConnectedToPart;
  private CfgItemList      fCfgOffRampConnectedToPart;
  private Label            fLabelNrOfLanesOnRamp;
  private CfgItemList      fCfgNrOfLanesOnRamp;
  private Label            fLabelNewNrOfLanes;
  private CfgItemList      fCfgNewNrOfLanes;
  private CfgInputBox      fCfgName;
  private Button           fOkButton;
  private boolean          fCanceled;
  private boolean          fFieldsDefined;

  public LinkBox(Frame parent, int DialogMode, Link link)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fDialogMode = DialogMode;
    fLink = link;
    fCanceled = true;
    fFieldsDefined = false;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    if (fDialogMode == kAddMode) {
      setTitle(fMessages.lookup("textAddLinkDialogTitle"));
    }
    else if (fDialogMode == kEditMode) {
      setTitle(fMessages.lookup("textEditLinkDialogTitle"));
    }
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // define the controlpanels
    Panel SettingsPanel = new Panel();
    Panel ControlPanel = new Panel();
    SettingsPanel.setLayout(new GridLayout(18,3));
    ControlPanel.setLayout(new GridLayout(1,2));

    fLabelID = new Label(fMessages.lookup("textLinkID") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelID);
    fCfgID = new CfgInputBox(kID);
    SettingsPanel.add(fCfgID);
    SettingsPanel.add(new Label());

    if (fDialogMode == kEditMode) {
      fLabelID.setEnabled(false);
      fCfgID.setEnabled(false);
    }

    SettingsPanel.add(new Label(fMessages.lookup("textLinkNrOfLanes") + " = ",Label.RIGHT));
    fCfgNrOfLanes = new CfgItemList(kNrOfLanes);
    SettingsPanel.add(fCfgNrOfLanes);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textLinkNodesOnLink") + " = ",Label.RIGHT));
    fCfgNodes = new CfgNodesInputBox();
    SettingsPanel.add(fCfgNodes);
    fLabelNodes = new Label(" (# = )",Label.LEFT);
    SettingsPanel.add(fLabelNodes);

    SettingsPanel.add(new Label(fMessages.lookup("textLinkSpeedLimit") + " = ",Label.RIGHT));
    fCfgSpeedLimit = new CfgScrollBar(kSpeedLimit);
    SettingsPanel.add(fCfgSpeedLimit);
    fLabelSpeedLimit = new Label( " (" + fMessages.lookup("textSpeedLimitUnit") + ")");
    SettingsPanel.add(fLabelSpeedLimit);

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    fLabelIsTerminator = new Label(fMessages.lookup("textLinkIsTerminator") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelIsTerminator);
    fCfgIsTerminator = new CfgCheckBox(kIsTerminator);
    SettingsPanel.add(fCfgIsTerminator);
    SettingsPanel.add(new Label());

    fLabelMainLinkConnectedToLink = new Label(fMessages.lookup("textLinkMainLinkConnectedToLink") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelMainLinkConnectedToLink);
    fCfgMainLinkConnectedToLink = new CfgInputBox(kMainLinkConnectedToLink);
    SettingsPanel.add(fCfgMainLinkConnectedToLink);
    SettingsPanel.add(new Label());

    fLabelMainLinkConnectedToPart = new Label(fMessages.lookup("textLinkMainLinkConnectedToPart") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelMainLinkConnectedToPart);
    fCfgMainLinkConnectedToPart = new CfgItemList(kMainLinkConnectedToPart);
    SettingsPanel.add(fCfgMainLinkConnectedToPart);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textLinkType") + " = ",Label.RIGHT));
    fCfgType = new CfgItemList(kType);
    SettingsPanel.add(fCfgType);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    fLabelLocation = new Label(fMessages.lookup("textLinkLocation") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelLocation);
    fCfgLocation = new CfgItemList(kLocation);
    SettingsPanel.add(fCfgLocation);
    SettingsPanel.add(new Label());

    fLabelOffRampConnectedToLink = new Label(fMessages.lookup("textLinkOffRampConnectedToLink") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelOffRampConnectedToLink);
    fCfgOffRampConnectedToLink = new CfgInputBox(kOffRampConnectedToLink);
    SettingsPanel.add(fCfgOffRampConnectedToLink);
    SettingsPanel.add(new Label());

    fLabelOffRampConnectedToPart = new Label(fMessages.lookup("textLinkOffRampConnectedToPart") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelOffRampConnectedToPart);
    fCfgOffRampConnectedToPart = new CfgItemList(kOffRampConnectedToPart);
    SettingsPanel.add(fCfgOffRampConnectedToPart);
    SettingsPanel.add(new Label());

    fLabelNrOfLanesOnRamp = new Label(fMessages.lookup("textLinkNrOfLanesOnRamp") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelNrOfLanesOnRamp);
    fCfgNrOfLanesOnRamp = new CfgItemList(kNrOfLanesOnRamp);
    SettingsPanel.add(fCfgNrOfLanesOnRamp);
    SettingsPanel.add(new Label());

    fLabelNewNrOfLanes = new Label(fMessages.lookup("textLinkNewNrOfLanes") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelNewNrOfLanes);
    fCfgNewNrOfLanes = new CfgItemList(kNewNrOfLanes);
    SettingsPanel.add(fCfgNewNrOfLanes);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textLinkName") + " = ",Label.RIGHT));
    fCfgName = new CfgInputBox(kName);
    SettingsPanel.add(fCfgName);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    fOkButton = new Button(fMessages.lookup("buttonOk"));
    fOkButton.addActionListener(this);
    ControlPanel.add(fOkButton);

    // add the controlpanel
    setLayout(new BorderLayout());
    add(SettingsPanel,"North");
    add(ControlPanel,"South");

    // add the windowlistener
    addWindowListener(this);

    fFieldsDefined = true;
    updateAllFields();

    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public Link getLink()
  {
    return fLink;
  }

  private void updateAllFields()
  {
    fCfgID.update();
    fCfgNrOfLanes.update();
    fCfgNodes.update();
    fCfgSpeedLimit.update();
    fCfgIsTerminator.update();
    fCfgMainLinkConnectedToLink.update();
    fCfgMainLinkConnectedToPart.update();
    fCfgType.update();
    fCfgLocation.update();
    fCfgOffRampConnectedToLink.update();
    fCfgOffRampConnectedToPart.update();
    fCfgNrOfLanesOnRamp.update();
    fCfgNewNrOfLanes.update();
    fCfgName.update();

    if (fLink.fNrOfNodes < 2) {
      fOkButton.setEnabled(false);
    }
    else {
      fOkButton.setEnabled(true);
    }

    // enable or disable the appropriate fields
    if (fFieldsDefined) {
      switch (fLink.fType) {
        case Link.kTunnel           :
        case Link.kHighway          : fLabelLocation.setEnabled(false);
                                      fCfgLocation.setEnabled(false);
                                      fLabelOffRampConnectedToLink.setEnabled(false);
                                      fCfgOffRampConnectedToLink.setEnabled(false);
                                      fLabelOffRampConnectedToPart.setEnabled(false);
                                      fCfgOffRampConnectedToPart.setEnabled(false);
                                      fLabelNrOfLanesOnRamp.setEnabled(false);
                                      fCfgNrOfLanesOnRamp.setEnabled(false);
                                      fLabelNewNrOfLanes.setEnabled(false);
                                      fCfgNewNrOfLanes.setEnabled(false);
                                      break;
        case Link.kSeparatedOnRamp  :
        case Link.kMergedOnRamp     :
        case Link.kSeparatedOffRamp :
        case Link.kMergedOffRamp    :
        case Link.kNarrowing        :
        case Link.kWidening         : fLabelLocation.setEnabled(true);
                                      fCfgLocation.setEnabled(true);
                                      fLabelOffRampConnectedToLink.setEnabled(true);
                                      fCfgOffRampConnectedToLink.setEnabled(true);
                                      fLabelOffRampConnectedToPart.setEnabled(true);
                                      fCfgOffRampConnectedToPart.setEnabled(true);
                                      fLabelNrOfLanesOnRamp.setEnabled(true);
                                      fCfgNrOfLanesOnRamp.setEnabled(true);
                                      fLabelNewNrOfLanes.setEnabled(true);
                                      fCfgNewNrOfLanes.setEnabled(true);
                                      break;
      }

      if ((fLink.fType == Link.kTunnel) ||
          (fLink.fType == Link.kHighway) ||
          (fLink.fType == Link.kNarrowing) ||
          (fLink.fType == Link.kWidening)) {
        fLabelIsTerminator.setEnabled(true);
        fCfgIsTerminator.setEnabled(true);
        if (fLink.fIsTerminator) {
          fLabelMainLinkConnectedToLink.setEnabled(false);
          fCfgMainLinkConnectedToLink.setEnabled(false);
          fLabelMainLinkConnectedToPart.setEnabled(false);
          fCfgMainLinkConnectedToPart.setEnabled(false);
        }
        else {
          fLabelMainLinkConnectedToLink.setEnabled(true);
          fCfgMainLinkConnectedToLink.setEnabled(true);
          fLabelMainLinkConnectedToPart.setEnabled(true);
          fCfgMainLinkConnectedToPart.setEnabled(true);
        }
      }
      else {
        if ((fLink.fType == Link.kSeparatedOnRamp) ||
            (fLink.fType == Link.kMergedOnRamp) ||
            (fLink.fType == Link.kSeparatedOffRamp) ||
            (fLink.fType == Link.kMergedOffRamp)) {
          fLabelIsTerminator.setEnabled(false);
          fCfgIsTerminator.setEnabled(false);
          fLabelMainLinkConnectedToLink.setEnabled(true);
          fCfgMainLinkConnectedToLink.setEnabled(true);
          fLabelMainLinkConnectedToPart.setEnabled(true);
          fCfgMainLinkConnectedToPart.setEnabled(true);
        }
      }

      if ((fLink.fType == Link.kNarrowing) ||
          (fLink.fType == Link.kWidening)) {
        fLabelLocation.setEnabled(true);
        fCfgLocation.setEnabled(true);
        fLabelOffRampConnectedToLink.setEnabled(false);
        fCfgOffRampConnectedToLink.setEnabled(false);
        fLabelOffRampConnectedToPart.setEnabled(false);
        fCfgOffRampConnectedToPart.setEnabled(false);
        fLabelNrOfLanesOnRamp.setEnabled(false);
        fCfgNrOfLanesOnRamp.setEnabled(false);
        fLabelNewNrOfLanes.setEnabled(true);
        fCfgNewNrOfLanes.setEnabled(true);
      }

      if ((fLink.fType == Link.kSeparatedOnRamp) ||
          (fLink.fType == Link.kMergedOnRamp) ||
          (fLink.fType == Link.kSeparatedOffRamp) ||
          (fLink.fType == Link.kMergedOffRamp)) {
        fLabelNewNrOfLanes.setEnabled(false);
        fCfgNewNrOfLanes.setEnabled(false);        

        if ((fLink.fType == Link.kSeparatedOnRamp) ||
            (fLink.fType == Link.kMergedOnRamp)) {
          fLabelOffRampConnectedToLink.setEnabled(false);
          fCfgOffRampConnectedToLink.setEnabled(false);
          fLabelOffRampConnectedToPart.setEnabled(false);
          fCfgOffRampConnectedToPart.setEnabled(false);
        }
      }
    }
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kID) {
        setText(String.valueOf(fLink.fID));
      }
      else if (fInputBox == kMainLinkConnectedToLink) {
        setText(String.valueOf(fLink.fMainLinkConnectedToLink));
      }
      else if (fInputBox == kOffRampConnectedToLink) {
        setText(String.valueOf(fLink.fOffRampConnectedToLink));
      }
      else if (fInputBox == kName) {
        setText(String.valueOf(fLink.fName));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int IntInput = 0;
      String StringInput = "";
      try {
        if (fInputBox != kName) {
          IntInput = (new Integer(e.getActionCommand())).intValue();
        }
        else {
          StringInput = e.getActionCommand();
        }
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kID) {
          IntInput = fLink.fID;
        }
        else if (fInputBox == kMainLinkConnectedToLink) {
          IntInput = fLink.fMainLinkConnectedToLink;
        }
        else if (fInputBox == kOffRampConnectedToLink) {
          IntInput = fLink.fOffRampConnectedToLink;
        }
      }

      if (fInputBox == kID) {
        // check if the ID is free
        Link CheckLink = (Link) fResources.fNetwork.fLinks.get(new Integer(IntInput));
        if (CheckLink == null) {
          // ok, the link is still free
          fLink.fID = IntInput;
        }
        else {
          // error, the link is already in use, don't change the ID
        }
      }
      else if (fInputBox == kMainLinkConnectedToLink) {
        fLink.fMainLinkConnectedToLink = IntInput;
      }
      else if (fInputBox == kOffRampConnectedToLink) {
        fLink.fOffRampConnectedToLink = IntInput;
      }
      else if (fInputBox == kName) {
        fLink.fName = StringInput;
      }

      updateAllFields();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgNodesInputBox extends TextField implements ActionListener
  {
    public CfgNodesInputBox()
    {
      super();
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      // concatenate all the node-IDs
      String AllIDs = "";
      for (int i = 0; i < fLink.fNrOfNodes; ++i) {
        AllIDs = AllIDs.concat(String.valueOf(fLink.fNodes[i].fID));
        if (i < (fLink.fNrOfNodes - 1)) {
          AllIDs = AllIDs.concat(",");
        }
      }
      setText(AllIDs);
      if (fFieldsDefined) {
        fLabelNodes.setText(" (# = " + String.valueOf(fLink.fNrOfNodes) + ")");
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      String Input = e.getActionCommand();
      String Result = "";

      // to temporary store the IDs
      Vector Nodes = new Vector();

      String ID = "";
      // parse the ID-string and extract all IDs
      for (int Index = 0; Index < Input.length(); ++Index) {
        char ch = Input.charAt(Index);
        if (ch != ',') {
          ID = ID.concat(String.valueOf(ch));
        }
        else {
          // we have completed extraction of one ID
          // test if it is a valid number
          try {
            int NodeID = (new Integer(ID)).intValue();
            // test if the specified node exists
            if (fResources.fNetwork.fNodes.get(new Integer(NodeID)) != null) {
              // add the ID to the result
              Result = Result.concat(ID + ",");
              // add the ID to the list of nodes
              Nodes.addElement(new Integer(ID));
            }
            else {
              // the specified node doesn't exist so we ignore it
            }
          }
          catch (NumberFormatException exc) {
            // ignore the ID
          }
          ID = "";
        }
      }

      // we have completed extraction of the last ID
      // test if it is a valid number
      try {
        int NodeID = (new Integer(ID)).intValue();
        // test if the specified node exists
        if (fResources.fNetwork.fNodes.get(new Integer(NodeID)) != null) {
          // add the ID to the result
          Result = Result.concat(ID + ",");
          // add the ID to the list of nodes
          Nodes.addElement(new Integer(ID));
        }
        else {
          // the specified node doesn't exist so we ignore it
        }
      }
      catch (NumberFormatException exc) {
        // ignore the ID
      }

      // delete the final comma
      if (Result.endsWith(",")) {
        Result = Result.substring(0,Result.length() - 1);
      }

      // modify the Nodes[]-field in fLink
      fLink.fNrOfNodes = Nodes.size();
      fLink.fNodes = new Node[fLink.fNrOfNodes];
      Enumeration NodeEnum = Nodes.elements();
      int NodeIndex = 0;
      while (NodeEnum.hasMoreElements()) {
        fLink.fNodes[NodeIndex] = (Node) fResources.fNetwork.fNodes.get((Integer) NodeEnum.nextElement());
        ++NodeIndex;
      }

      fLink.fLengths = new int[fLink.fNrOfNodes - 1];
      fLink.fGrades = new float[fLink.fNrOfNodes - 1];
      for (int i = 0; i < (fLink.fNrOfNodes - 1); ++i) {
        fLink.fLengths[i] = 0;
        fLink.fGrades[i] = 0;
      }

      updateAllFields();
    }

    public Dimension getPreferredSize()
    {
      return new Dimension(150,25);
    }

    public Dimension getMinimumSize()
    {
      return getPreferredSize();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgCheckBox extends Checkbox implements ItemListener
  {
    private int fCheckBox;

    public CfgCheckBox(int checkBox)
    {
      super();
      fCheckBox = checkBox;
      addItemListener(this);
      update();
    }

    synchronized public void update()
    {
      if (fCheckBox == kIsTerminator) {
        setState(fLink.fIsTerminator);
        if (fLink.fIsTerminator) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fCheckBox == kIsTerminator) {
        fLink.fIsTerminator = !fLink.fIsTerminator;
      }

      updateAllFields();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgItemList extends Choice implements ItemListener
  {
    private int fItemList;

    public CfgItemList(int itemList)
    {
      super();
      fItemList = itemList;
      if ((fItemList == kNrOfLanes) ||
          (fItemList == kNrOfLanesOnRamp) ||
          (fItemList == kNewNrOfLanes)) {
        add("1");
        add("2");
        add("3");
        add("4");
        add("5");
        add("6");
        add("7");
        add("8");
        add("9");
        add("10");
        add("11");
        add("12");
      }
      else if ((fItemList == kMainLinkConnectedToPart) ||
               (fItemList == kOffRampConnectedToPart)) {
        add(fMessages.lookup("textMainLinkConnection"));
        add(fMessages.lookup("textOnRampConnection"));
      }
      else if (fItemList == kType) {
        add(fMessages.lookup("textTunnelLink"));
        add(fMessages.lookup("textHighwayLink"));
        add(fMessages.lookup("textSeparatedOnRampLink"));
        add(fMessages.lookup("textMergedOnRampLink"));
        add(fMessages.lookup("textSeparatedOffRampLink"));
        add(fMessages.lookup("textMergedOffRampLink"));
        add(fMessages.lookup("textNarrowingLink"));
        add(fMessages.lookup("textWideningLink"));
      }
      else if (fItemList == kLocation) {
        add(fMessages.lookup("textLeftLocation"));
        add(fMessages.lookup("textRightLocation"));
      }
      update();
      addItemListener(this);
    }

    synchronized public void update()
    {
      if (fItemList == kNrOfLanes) {
        if (fLink.fNrOfLanes == 1) {
          select("1");
        }
        else if (fLink.fNrOfLanes == 2) {
          select("2");
        }
        else if (fLink.fNrOfLanes == 3) {
          select("3");
        }
        else if (fLink.fNrOfLanes == 4) {
          select("4");
        }
        else if (fLink.fNrOfLanes == 5) {
          select("5");
        }
        else if (fLink.fNrOfLanes == 6) {
          select("6");
        }
        else if (fLink.fNrOfLanes == 7) {
          select("7");
        }
        else if (fLink.fNrOfLanes == 8) {
          select("8");
        }
        else if (fLink.fNrOfLanes == 9) {
          select("9");
        }
        else if (fLink.fNrOfLanes == 10) {
          select("10");
        }
        else if (fLink.fNrOfLanes == 11) {
          select("11");
        }
        else if (fLink.fNrOfLanes == 12) {
          select("12");
        }
      }
      else if (fItemList == kMainLinkConnectedToPart) {
        if (fLink.fMainLinkConnectedToPart == Link.kConnectedToMainLink) {
          select(fMessages.lookup("textMainLinkConnection"));
        }
        else if (fLink.fMainLinkConnectedToPart == Link.kConnectedToOnRamp) {
          select(fMessages.lookup("textOnRampConnection"));
        }
      }
      else if (fItemList == kType) {
        if (fLink.fType == Link.kTunnel) {
          select(fMessages.lookup("textTunnelLink"));
        }
        else if (fLink.fType == Link.kHighway) {
          select(fMessages.lookup("textHighwayLink"));
        }
        else if (fLink.fType == Link.kSeparatedOnRamp) {
          select(fMessages.lookup("textSeparatedOnRampLink"));
        }
        else if (fLink.fType == Link.kMergedOnRamp) {
          select(fMessages.lookup("textMergedOnRampLink"));
        }
        else if (fLink.fType == Link.kSeparatedOffRamp) {
          select(fMessages.lookup("textSeparatedOffRampLink"));
        }
        else if (fLink.fType == Link.kMergedOffRamp) {
          select(fMessages.lookup("textMergedOffRampLink"));
        }
        else if (fLink.fType == Link.kNarrowing) {
          select(fMessages.lookup("textNarrowingLink"));
        }
        else if (fLink.fType == Link.kWidening) {
          select(fMessages.lookup("textWideningLink"));
        }
      }
      else if (fItemList == kLocation) {
        if (fLink.fLocation == Link.kLeft) {
          select(fMessages.lookup("textLeftLocation"));
        }
        else if (fLink.fLocation == Link.kRight) {
          select(fMessages.lookup("textRightLocation"));
        }
      }
      else if (fItemList == kOffRampConnectedToPart) {
        if (fLink.fOffRampConnectedToPart == Link.kConnectedToMainLink) {
          select(fMessages.lookup("textMainLinkConnection"));
        }
        else if (fLink.fOffRampConnectedToPart == Link.kConnectedToOnRamp) {
          select(fMessages.lookup("textOnRampConnection"));
        }
      }
      else if (fItemList == kNrOfLanesOnRamp) {
        if (fLink.fNrOfLanesOnRamp == 1) {
          select("1");
        }
        else if (fLink.fNrOfLanesOnRamp == 2) {
          select("2");
        }
        else if (fLink.fNrOfLanesOnRamp == 3) {
          select("3");
        }
        else if (fLink.fNrOfLanesOnRamp == 4) {
          select("4");
        }
        else if (fLink.fNrOfLanesOnRamp == 5) {
          select("5");
        }
        else if (fLink.fNrOfLanesOnRamp == 6) {
          select("6");
        }
        else if (fLink.fNrOfLanesOnRamp == 7) {
          select("7");
        }
        else if (fLink.fNrOfLanesOnRamp == 8) {
          select("8");
        }
        else if (fLink.fNrOfLanesOnRamp == 9) {
          select("9");
        }
        else if (fLink.fNrOfLanesOnRamp == 10) {
          select("10");
        }
        else if (fLink.fNrOfLanesOnRamp == 11) {
          select("11");
        }
        else if (fLink.fNrOfLanesOnRamp == 12) {
          select("12");
        }
      }
      else if (fItemList == kNewNrOfLanes) {
        if (fLink.fNewNrOfLanes == 1) {
          select("1");
        }
        else if (fLink.fNewNrOfLanes == 2) {
          select("2");
        }
        else if (fLink.fNewNrOfLanes == 3) {
          select("3");
        }
        else if (fLink.fNewNrOfLanes == 4) {
          select("4");
        }
        else if (fLink.fNewNrOfLanes == 5) {
          select("5");
        }
        else if (fLink.fNewNrOfLanes == 6) {
          select("6");
        }
        else if (fLink.fNewNrOfLanes == 7) {
          select("7");
        }
        else if (fLink.fNewNrOfLanes == 8) {
          select("8");
        }
        else if (fLink.fNewNrOfLanes == 9) {
          select("9");
        }
        else if (fLink.fNewNrOfLanes == 10) {
          select("10");
        }
        else if (fLink.fNewNrOfLanes == 11) {
          select("11");
        }
        else if (fLink.fNewNrOfLanes == 12) {
          select("12");
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fItemList == kNrOfLanes) {
        String LinkNrOfLanes = (String) e.getItem();

        if (LinkNrOfLanes.equalsIgnoreCase("1")) {
          fLink.fNrOfLanes = 1;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("2")) {
          fLink.fNrOfLanes = 2;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("3")) {
          fLink.fNrOfLanes = 3;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("4")) {
          fLink.fNrOfLanes = 4;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("5")) {
          fLink.fNrOfLanes = 5;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("6")) {
          fLink.fNrOfLanes = 6;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("7")) {
          fLink.fNrOfLanes = 7;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("8")) {
          fLink.fNrOfLanes = 8;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("9")) {
          fLink.fNrOfLanes = 9;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("10")) {
          fLink.fNrOfLanes = 10;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("11")) {
          fLink.fNrOfLanes = 11;
        }
        else if (LinkNrOfLanes.equalsIgnoreCase("12")) {
          fLink.fNrOfLanes = 12;
        }

        fLink.createDefaultLaneChangePermissions();
      }
      else if (fItemList == kMainLinkConnectedToPart) {
        String ConnectionType = (String) e.getItem();

        if (ConnectionType.equalsIgnoreCase(fMessages.lookup("textMainLinkConnection"))) {
          fLink.fMainLinkConnectedToPart = Link.kConnectedToMainLink;
        }
        else if (ConnectionType.equalsIgnoreCase(fMessages.lookup("textOnRampConnection"))) {
          fLink.fMainLinkConnectedToPart = Link.kConnectedToOnRamp;
        }
      }
      else if (fItemList == kType) {
        String LinkType = (String) e.getItem();

        if (LinkType.equalsIgnoreCase(fMessages.lookup("textTunnelLink"))) {
          fLink.fType = Link.kTunnel;
        }
        else if (LinkType.equalsIgnoreCase(fMessages.lookup("textHighwayLink"))) {
          fLink.fType = Link.kHighway;
        }
        else if (LinkType.equalsIgnoreCase(fMessages.lookup("textSeparatedOnRampLink"))) {
          fLink.fType = Link.kSeparatedOnRamp;
        }
        else if (LinkType.equalsIgnoreCase(fMessages.lookup("textMergedOnRampLink"))) {
          fLink.fType = Link.kMergedOnRamp;
        }
        else if (LinkType.equalsIgnoreCase(fMessages.lookup("textSeparatedOffRampLink"))) {
          fLink.fType = Link.kSeparatedOffRamp;
        }
        else if (LinkType.equalsIgnoreCase(fMessages.lookup("textMergedOffRampLink"))) {
          fLink.fType = Link.kMergedOffRamp;
        }
        else if (LinkType.equalsIgnoreCase(fMessages.lookup("textNarrowingLink"))) {
          fLink.fType = Link.kNarrowing;
        }
        else if (LinkType.equalsIgnoreCase(fMessages.lookup("textWideningLink"))) {
          fLink.fType = Link.kWidening;
        }

        fLink.createDefaultLaneChangePermissions();
      }
      else if (fItemList == kLocation) {
        String LinkLocation = (String) e.getItem();

        if (LinkLocation.equalsIgnoreCase(fMessages.lookup("textLeftLocation"))) {
          fLink.fLocation = Link.kLeft;
        }
        else if (LinkLocation.equalsIgnoreCase(fMessages.lookup("textRightLocation"))) {
          fLink.fLocation = Link.kRight;
        }

        fLink.createDefaultLaneChangePermissions();
      }
      else if (fItemList == kOffRampConnectedToPart) {
        String ConnectionType = (String) e.getItem();

        if (ConnectionType.equalsIgnoreCase(fMessages.lookup("textMainLinkConnection"))) {
          fLink.fOffRampConnectedToPart = Link.kConnectedToMainLink;
        }
        else if (ConnectionType.equalsIgnoreCase(fMessages.lookup("textOnRampConnection"))) {
          fLink.fOffRampConnectedToPart = Link.kConnectedToOnRamp;
        }
      }
      if (fItemList == kNrOfLanesOnRamp) {
        String LinkNrOfLanesOnRamp = (String) e.getItem();

        if (LinkNrOfLanesOnRamp.equalsIgnoreCase("1")) {
          fLink.fNrOfLanesOnRamp = 1;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("2")) {
          fLink.fNrOfLanesOnRamp = 2;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("3")) {
          fLink.fNrOfLanesOnRamp = 3;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("4")) {
          fLink.fNrOfLanesOnRamp = 4;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("5")) {
          fLink.fNrOfLanesOnRamp = 5;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("6")) {
          fLink.fNrOfLanesOnRamp = 6;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("7")) {
          fLink.fNrOfLanesOnRamp = 7;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("8")) {
          fLink.fNrOfLanesOnRamp = 8;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("9")) {
          fLink.fNrOfLanesOnRamp = 9;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("10")) {
          fLink.fNrOfLanesOnRamp = 10;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("11")) {
          fLink.fNrOfLanesOnRamp = 11;
        }
        else if (LinkNrOfLanesOnRamp.equalsIgnoreCase("12")) {
          fLink.fNrOfLanesOnRamp = 12;
        }

        fLink.createDefaultLaneChangePermissions();
      }
      if (fItemList == kNewNrOfLanes) {
        String LinkNewNrOfLanes = (String) e.getItem();

        if (LinkNewNrOfLanes.equalsIgnoreCase("1")) {
          fLink.fNewNrOfLanes = 1;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("2")) {
          fLink.fNewNrOfLanes = 2;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("3")) {
          fLink.fNewNrOfLanes = 3;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("4")) {
          fLink.fNewNrOfLanes = 4;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("5")) {
          fLink.fNewNrOfLanes = 5;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("6")) {
          fLink.fNewNrOfLanes = 6;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("7")) {
          fLink.fNewNrOfLanes = 7;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("8")) {
          fLink.fNewNrOfLanes = 8;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("9")) {
          fLink.fNewNrOfLanes = 9;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("10")) {
          fLink.fNewNrOfLanes = 10;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("11")) {
          fLink.fNewNrOfLanes = 11;
        }
        else if (LinkNewNrOfLanes.equalsIgnoreCase("12")) {
          fLink.fNewNrOfLanes = 12;
        }

        fLink.createDefaultLaneChangePermissions();
      }

      updateAllFields();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgScrollBar extends Scrollbar implements AdjustmentListener
  {
    private int fScrollBar;

    public CfgScrollBar(int ScrollBar)
    {
      super();
      setOrientation(Scrollbar.HORIZONTAL);
      fScrollBar = ScrollBar;
      if (fScrollBar == kSpeedLimit) {
        setValues(fLink.fSpeedLimit,50,1,300 + 50);
      }
      update();
      addAdjustmentListener(this);
    }

    synchronized public void update()
    {
      if (fScrollBar == kSpeedLimit) {
        setValue(fLink.fSpeedLimit);
        if (fLink.fSpeedLimit < 20) {
          setBackground(fResources.knm0To19SpeedColor);
        }
        else if (fLink.fSpeedLimit < 40) {
          setBackground(fResources.knm20To39SpeedColor);
        }
        else if (fLink.fSpeedLimit < 60) {
          setBackground(fResources.knm40To59SpeedColor);
        }
        else if (fLink.fSpeedLimit < 80) {
          setBackground(fResources.knm60To79SpeedColor);
        }
        else if (fLink.fSpeedLimit < 100) {
          setBackground(fResources.knm80To99SpeedColor);
        }
        else if (fLink.fSpeedLimit < 120) {
          setBackground(fResources.knm100To119SpeedColor);
        }
        else {
          setBackground(fResources.knm120AndUpSpeedColor);
        }

        if (fFieldsDefined) {
          fLabelSpeedLimit.setText(" (" +
                                   String.valueOf(fLink.fSpeedLimit) +
                                   " " +
                                   fMessages.lookup("textSpeedLimitUnit") +
                                   ")");
        }
      }
    }

    // method from the AdjustmentListener-interface
    public void adjustmentValueChanged(AdjustmentEvent e)
    {
      if (fScrollBar == kSpeedLimit) {
        fLink.fSpeedLimit = e.getValue();
      }

      updateAllFields();
    }
  }
}
