// ----------------------------------
// Filename      : NodeBox.java
// Created       : 20/12/1999
// Last modified : 08/02/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import mitrasim.network.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class NodeBox extends Dialog implements WindowListener, ActionListener
{
  // the different dialog-modes
  public static final int kAddMode  = 1;
  public static final int kEditMode = 2;

  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  private int         fDialogMode;
  private Node        fNode;
  private final int   kID        = 1;
  private final int   kXPosition = 2;
  private final int   kYPosition = 3;
  private final int   kZPosition = 4;
  private final int   kType      = 5;
  private final int   kName      = 6;
  private Label       fLabelID;
  private CfgInputBox fCfgID;
  private CfgInputBox fCfgXPosition;
  private CfgInputBox fCfgYPosition;
  private CfgInputBox fCfgZPosition;
  private CfgItemList fCfgType;
  private CfgInputBox fCfgName;
  private boolean     fCanceled;

  public NodeBox(Frame parent, int DialogMode, Node node)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fDialogMode = DialogMode;
    fNode = node;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    if (fDialogMode == kAddMode) {
      setTitle(fMessages.lookup("textAddNodeDialogTitle"));
    }
    else if (fDialogMode == kEditMode) {
      setTitle(fMessages.lookup("textEditNodeDialogTitle"));
    }
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // define the controlpanels
    Panel SettingsPanel = new Panel();
    Panel ControlPanel = new Panel();
    SettingsPanel.setLayout(new GridLayout(7,3));
    ControlPanel.setLayout(new GridLayout(1,2));

    fLabelID = new Label(fMessages.lookup("textNodeID") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelID);
    fCfgID = new CfgInputBox(kID);
    SettingsPanel.add(fCfgID);
    SettingsPanel.add(new Label());

    if (fDialogMode == kEditMode) {
      fLabelID.setEnabled(false);
      fCfgID.setEnabled(false);
    }

    SettingsPanel.add(new Label(fMessages.lookup("textNodeXPosition") + " = ",Label.RIGHT));
    fCfgXPosition = new CfgInputBox(kXPosition);
    SettingsPanel.add(fCfgXPosition);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textNodeYPosition") + " = ",Label.RIGHT));
    fCfgYPosition = new CfgInputBox(kYPosition);
    SettingsPanel.add(fCfgYPosition);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textNodeZPosition") + " = ",Label.RIGHT));
    fCfgZPosition = new CfgInputBox(kZPosition);
    SettingsPanel.add(fCfgZPosition);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textNodeType") + " = ",Label.RIGHT));
    fCfgType = new CfgItemList(kType);
    SettingsPanel.add(fCfgType);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textNodeName") + " = ",Label.RIGHT));
    fCfgName = new CfgInputBox(kName);
    SettingsPanel.add(fCfgName);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    button = new Button(fMessages.lookup("buttonOk"));
    button.addActionListener(this);
    ControlPanel.add(button);

    // add the controlpanel
    setLayout(new BorderLayout());
    add(SettingsPanel,"North");
    add(ControlPanel,"South");

    // add the windowlistener
    addWindowListener(this);

    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public Node getNode()
  {
    return fNode;
  }

  private void update()
  {
    fCfgID.update();
    fCfgXPosition.update();
    fCfgYPosition.update();
    fCfgZPosition.update();
    fCfgType.update();
    fCfgName.update();
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kID) {
        setText(String.valueOf(fNode.fID));
      }
      else if (fInputBox == kXPosition) {
        setText(String.valueOf(fNode.fXPos));
      }
      else if (fInputBox == kYPosition) {
        setText(String.valueOf(fNode.fYPos));
      }
      else if (fInputBox == kZPosition) {
        setText(String.valueOf(fNode.fZPos));
      }
      else if (fInputBox == kName) {
        setText(String.valueOf(fNode.fName));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int IntInput = 0;
      String StringInput = "";
      try {
        if (fInputBox != kName) {
          IntInput = (new Integer(e.getActionCommand())).intValue();
        }
        else {
          StringInput = e.getActionCommand();
        }
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kID) {
          IntInput = fNode.fID;
        }
        else if (fInputBox == kXPosition) {
          IntInput = fNode.fXPos;
        }
        else if (fInputBox == kYPosition) {
          IntInput = fNode.fYPos;
        }
        else if (fInputBox == kZPosition) {
          IntInput = fNode.fZPos;
        }
      }

      if (fInputBox == kID) {
        // check if the ID is free
        Node CheckNode = (Node) fResources.fNetwork.fNodes.get(new Integer(IntInput));
        if (CheckNode == null) {
          // ok, the node is still free
          fNode.fID = IntInput;
        }
        else {
          // error, the node is already in use, don't change the ID
        }
      }
      else if (fInputBox == kXPosition) {
        fNode.fXPos = IntInput;
      }
      else if (fInputBox == kYPosition) {
        fNode.fYPos = IntInput;
      }
      else if (fInputBox == kZPosition) {
        fNode.fZPos = IntInput;
      }
      else if (fInputBox == kName) {
        fNode.fName = StringInput;
      }

      update();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgItemList extends Choice implements ItemListener
  {
    private int fItemList;

    public CfgItemList(int itemList)
    {
      super();
      fItemList = itemList;
      if (fItemList == kType) {
        add(fMessages.lookup("textOriginNode"));
        add(fMessages.lookup("textDestinationNode"));
        add(fMessages.lookup("textJunctionNode"));
        add(fMessages.lookup("textBendNode"));
        add(fMessages.lookup("textEnvironmentNode"));
      }
      update();
      addItemListener(this);
    }

    synchronized public void update()
    {
      if (fItemList == kType) {
        if (fNode.fType == Node.kOrigin) {
          select(fMessages.lookup("textOriginNode"));
        }
        else if (fNode.fType == Node.kDestination) {
          select(fMessages.lookup("textDestinationNode"));
        }
        else if (fNode.fType == Node.kJunction) {
          select(fMessages.lookup("textJunctionNode"));
        }
        else if (fNode.fType == Node.kBend) {
          select(fMessages.lookup("textBendNode"));
        }
        else if (fNode.fType == Node.kEnvironment) {
          select(fMessages.lookup("textEnvironmentNode"));
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fItemList == kType) {
        String NodeType = (String) e.getItem();

        if (NodeType.equalsIgnoreCase(fMessages.lookup("textOriginNode"))) {
          fNode.fType = Node.kOrigin;
        }
        else if (NodeType.equalsIgnoreCase(fMessages.lookup("textDestinationNode"))) {
          fNode.fType = Node.kDestination;
        }
        else if (NodeType.equalsIgnoreCase(fMessages.lookup("textJunctionNode"))) {
          fNode.fType = Node.kJunction;
        }
        else if (NodeType.equalsIgnoreCase(fMessages.lookup("textBendNode"))) {
          fNode.fType = Node.kBend;
        }
        else if (NodeType.equalsIgnoreCase(fMessages.lookup("textEnvironmentNode"))) {
          fNode.fType = Node.kEnvironment;
        }
      }
    }
  }
}
