// ----------------------------------
// Filename      : Environment.java
// Created       : 06/11/1999
// Last modified : 27/04/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.network;

import java.awt.*;
import mitrasim.network.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class Environment
{
  // the different environment-types
  public static final short kRectangle = 1;
  public static final short kPolygon   = 2;
  public static final short kOval      = 3;
  public static final short kRailroad  = 4;

  public short       fType;
  public Color       fColor;
  public boolean     fFilled;
  public int         fNrOfNodes;
  public Node[]      fNodes;
  public int         fXRadius;
  public int         fYRadius;
  public BoundingBox fBoundingBox;
  public String      fName;

  public Environment()
  {
    fType = kRectangle;
    fColor = Color.black;
    fFilled = false;
    fNrOfNodes = 0;
    fNodes = null;
    fXRadius = 0;
    fYRadius = 0;
    fBoundingBox = new BoundingBox();
    fName = "";
  }

  public String translate(Messages messages)
  {
    if (fType == kRectangle) {
      return messages.lookup("textRectangleEnvironment");
    }
    else if (fType == kPolygon) {
      return messages.lookup("textPolygonEnvironment");
    }
    else if (fType == kOval) {
      return messages.lookup("textOvalEnvironment");
    }
    else if (fType == kRailroad) {
      return messages.lookup("textRailroadEnvironment");
    }
    return ""; // the compiler complains without this final return-statement
  }

  public void show()
  {
    System.out.println("Environment : type=" + " " + fType + "[" + fName + "]");
  }

  public void calcBoundingBox()
  {
    if (fNrOfNodes == 0) {
      return;
    }

    if ((fType == kRectangle) ||
        (fType == kPolygon) ||
        (fType == kRailroad)) {

      for (int i = 0; i < fNrOfNodes; ++i) {
        if (i == 0) {
          fBoundingBox.fMinX = fNodes[0].fXPos;
          fBoundingBox.fMinY = fNodes[0].fYPos;
          fBoundingBox.fMaxX = fNodes[0].fXPos;
          fBoundingBox.fMaxY = fNodes[0].fYPos;
        }
        else {
          if (fNodes[i].fXPos < fBoundingBox.fMinX) {
            fBoundingBox.fMinX = fNodes[i].fXPos;
          }
          if (fNodes[i].fYPos < fBoundingBox.fMinY) {
            fBoundingBox.fMinY = fNodes[i].fYPos;
          }
          if (fNodes[i].fXPos > fBoundingBox.fMaxX) {
            fBoundingBox.fMinX = fNodes[i].fXPos;
          }
          if (fNodes[i].fYPos > fBoundingBox.fMaxY) {
            fBoundingBox.fMinY = fNodes[i].fYPos;
          }
        }
      }

      fBoundingBox.fMinZ = fNodes[0].fZPos;
      fBoundingBox.fMaxZ = fBoundingBox.fMinZ;
    }
    else if (fType == kOval) {
      fBoundingBox.fMinX = fNodes[0].fXPos - fXRadius;
      fBoundingBox.fMinY = fNodes[0].fYPos - fYRadius;
      fBoundingBox.fMaxX = fNodes[0].fXPos + fXRadius;
      fBoundingBox.fMaxY = fNodes[0].fYPos + fYRadius;
      fBoundingBox.fMinZ = fNodes[0].fZPos;
      fBoundingBox.fMaxZ = fBoundingBox.fMinZ;
    }

    fBoundingBox.fUndefined = false;
  }

  public void cloneTo(Environment duplicate)
  {
    duplicate.fType = fType;
    duplicate.fColor = fColor;
    duplicate.fFilled = fFilled;
    duplicate.fNrOfNodes = fNrOfNodes;
    duplicate.fNodes = new Node[fNrOfNodes];
    if (fNodes != null) {
      for (int i = 0; i < fNrOfNodes; ++i) {
        duplicate.fNodes[i] = fNodes[i];
      }
    }
    duplicate.fXRadius = fXRadius;
    duplicate.fYRadius = fYRadius;
    fBoundingBox.cloneTo(duplicate.fBoundingBox);
    duplicate.fName = new String(fName);
  }

  public boolean nodeCompliant()
  {
    return (((fType == kRectangle) && (fNrOfNodes == 2)) ||
            ((fType == kPolygon)   && (fNrOfNodes >= 3)) ||
            ((fType == kOval)      && (fNrOfNodes == 1)) ||
            ((fType == kRailroad)  && (fNrOfNodes >= 2)));
  }
}
