// ----------------------------------
// Filename      : Node.java
// Created       : 08/10/1999
// Last modified : 11/02/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.network;

import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class Node
{
  // the different node-types
  public static final short kOrigin      = 1;
  public static final short kDestination = 2;
  public static final short kJunction    = 3;
  public static final short kBend        = 4;
  public static final short kEnvironment = 5;

  public int    fID;
  public int    fXPos;
  public int    fYPos;
  public int    fZPos;
  public short  fType;
  public String fName;

  public Node()
  {
    fID = 0;
    fXPos = 0;
    fYPos = 0;
    fZPos = 0;
    fType = kOrigin;
    fName = "";
  }

  public void set(int x, int y, int z)
  {
    fXPos = x;
    fYPos = y;
    fZPos = z;
  }

  public String translate(Messages messages)
  {
    if (fType == kOrigin) {
      return messages.lookup("textOriginNode");
    }
    else if (fType == kDestination) {
      return messages.lookup("textDestinationNode");
    }
    else if (fType == kJunction) {
      return messages.lookup("textJunctionNode");
    }
    else if (fType == kBend) {
      return messages.lookup("textBendNode");
    }
    else if (fType == kEnvironment) {
      return messages.lookup("textEnvironmentNode");
    }
    return ""; // the compiler complains without this final return-statement
  }

  public void show()
  {
    System.out.println("Node #" + fID + " : (" + fXPos + "," + fYPos +
                       "," + fZPos + ") type=" + " " + fType + "[" + fName + "]");
  }

  public void cloneTo(Node duplicate)
  {
    duplicate.fID = fID;
    duplicate.fXPos = fXPos;
    duplicate.fYPos = fYPos;
    duplicate.fZPos = fZPos;
    duplicate.fType = fType;
    duplicate.fName = new String(fName);
  }
}
