// ----------------------------------------
// Filename      : SetCycleHoldTimeBox.java
// Created       : 28/05/2000
// Last modified : 02/06/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------------

package mitrasim.remotecontrol;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class SetCycleHoldTimeBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  private final int   kCycleHoldTime = 1;
  private CfgInputBox fCfgCycleHoldTime;
  private int         fOldCycleHoldTime;
  private boolean     fCanceled;

  public SetCycleHoldTimeBox(Frame parent)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fOldCycleHoldTime = fResources.fsimCycleHoldTime;

    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textSetCycleHoldTimeBoxTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // create the configuration-panel
    Panel CfgPanel = new Panel();
    CfgPanel.setLayout(new GridLayout(2,3));

    CfgPanel.add(new Label(fMessages.lookup("textCycleHoldTime") + " : ",Label.RIGHT));
    fCfgCycleHoldTime = new CfgInputBox(kCycleHoldTime);
    CfgPanel.add(fCfgCycleHoldTime);
    CfgPanel.add(new Label(" " + fMessages.lookup("textMsUnit"),Label.LEFT));

    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));

    // create the control-panel
    Panel ControlPanel = new Panel();
    ControlPanel.setLayout(new GridLayout(1,2));

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    button = new Button(fMessages.lookup("buttonOk"));
    button.addActionListener(this);
    ControlPanel.add(button);

    setLayout(new BorderLayout());
    add("North",CfgPanel);
    add("South",ControlPanel);

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    if (fCanceled) {
      fResources.fsimCycleHoldTime = fOldCycleHoldTime;
    }
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kCycleHoldTime) {
        setText(String.valueOf(fResources.fsimCycleHoldTime));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int NewInput = 0;
      try {
        NewInput = (new Integer(e.getActionCommand())).intValue();
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kCycleHoldTime) {
          NewInput = fResources.fsimCycleHoldTime;
        }
      }

      if (fInputBox == kCycleHoldTime) {
        if (NewInput < 0) {
          NewInput = 0;
        }
        fResources.fsimCycleHoldTime = NewInput;
      }

      update();
    }
  }
}
