// ----------------------------------
// Filename      : Server.java
// Created       : 16/02/2000
// Last modified : 02/05/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.server;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import mitrasim.resources.*;
import mitrasim.simulator.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class Server
{
  // internal datastructures
  private Resources    fResources;
  private Messages     fMessages;
  private int          fServerPort;
  private InetAddress  fServerIP;
  private ServerSocket fServerSocket;
  private Simulator    fSimulator;
  private Thread       fSimulatorThread;

  public Server(int ServerPort)
  {
    fResources = new Resources();
    fMessages = fResources.fMessages;

    fServerPort = ServerPort;

    fServerIP = null;
    try {
      fServerIP = InetAddress.getLocalHost();
    }
    catch (UnknownHostException exc) {
      showMessage(fMessages.lookup("errorServerIPUnavailable"));
    }

    if (fServerIP != null) {
      showMessage(fMessages.lookup("messageServerHasIPAddress",
                                   String.valueOf(fServerIP)));
    }
    showMessage(fMessages.lookup("messageServerListeningOnTCPPort",
                                 String.valueOf(fServerPort)));
  }

  public void activate()
  {
    try {
      fServerSocket = new ServerSocket(fServerPort);
    }
    catch (IOException exc) {
      showMessage(fMessages.lookup("errorServerFailedToListenOnTCPPort",
                                   String.valueOf(fServerPort)));
      return;
    }

    // create the simulator
    fSimulator = new Simulator();

    // create the simulator's thread
    fSimulatorThread = new Thread(fSimulator);
    fSimulatorThread.start();

    // create the server's log
    fResources.fServerLog = new Report();
    String ReportString = "";

    // get the IP-address of the server
    if (fServerIP != null) {
      ReportString = fMessages.lookup("messageServerHasIPAddress",
                                      String.valueOf(fServerIP));
    }
    else {
      ReportString = fMessages.lookup("errorServerIPUnavailable");
    } 

    // add it to the connection-logbook
    fResources.fServerLog.addLine(ReportString);

    // get the TCP-port of the server
    ReportString = fMessages.lookup("messageServerListeningOnTCPPort",
                                    String.valueOf(fServerPort));

    // add it to the connection-logbook
    fResources.fServerLog.addLine(ReportString);

    // get the date and time of the server's activation
    Date CurrentDate = new Date();
    String Timestamp = DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.MEDIUM).format(CurrentDate);
    ReportString = fMessages.lookup("messageServerOnlineSince",Timestamp);

    // add it to the connection-logbook
    fResources.fServerLog.addLine(ReportString);

    // handle all incoming connections from clients
    while (true) {
      try {
        // accept clients' connections
        Socket socket = fServerSocket.accept();
        showMessage(fMessages.lookup("messageServerConnectionEstablished",
                                     String.valueOf(socket.getInetAddress())));

        ServerSocketHandler SocketHandler = new ServerSocketHandler(socket);

        // get the date and time of connection
        CurrentDate = new Date();
        Timestamp = DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.MEDIUM).format(CurrentDate);
        ReportString = "";

        if (SocketHandler.isValidProtocol()) {

          // handle the connection in a separate thread
          Thread thread = new Thread(SocketHandler);
          thread.start();

          ReportString = fMessages.lookup("messageLogSuccessfulConnection",
                                          Timestamp,
                                          String.valueOf(socket.getInetAddress()),
                                          SocketHandler.getProtocolAbbreviation());
        }
        else {
          ReportString = fMessages.lookup("messageLogFailedConnection",
                                          Timestamp,
                                          String.valueOf(socket.getInetAddress()));
        }
        // log the connection
        fResources.fServerLog.addLine(ReportString);
      }
      catch (IOException exc) {
        showMessage(fMessages.lookup("errorServerConnectionNotEstablished"));
      }
    }
  }

  private void showMessage(String message)
  {
    System.out.println("[server] " + message);
  }
}
