// ----------------------------------
// Filename      : ViewerGUI.java
// Created       : 20/10/1999
// Last modified : 02/05/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.viewer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import mitrasim.engine3d.*;
import mitrasim.network.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class ViewerGUI extends Frame implements WindowListener, ActionListener, KeyListener
{
  // internal datastructures
  private Resources      fResources;
  private int            fWinXSize;
  private int            fWinYSize;
  private Messages       fMessages;
  private Network        fNetwork;
  private ViewerCanvas   fViewerCanvas;
  private float          fWorldX1;
  private float          fWorldY1;
  private float          fWorldX2;
  private float          fWorldY2;
  private Renderer       fRenderer;
  private boolean        fAboutBoxShown;
  private final short    kNone             = 0;
  private final short    kRotateXYLeft     = 1;
  private final short    kRotateXYRight    = 2;
  private final short    kRotateZLeft      = 3;
  private final short    kRotateZRight     = 4;
  private final short    kIncreaseDistance = 5;
  private final short    kDecreaseDistance = 6;
  private short          fLastAction;

  public ViewerGUI()
  {
    super();

    fResources = new Resources();
    fWinXSize = fResources.fnvXSize;
    fWinYSize = fResources.fnvYSize;
    fMessages = fResources.fMessages;
    fNetwork = fResources.fNetwork;
    fWorldX1 = -0.5f;
    fWorldY1 = -0.5f;
    fWorldX2 = 0.5f;
    fWorldY2 = 0.5f;

    // initialize the 3D-renderer
    fResources.fnvViewPoint = new Point3D(0.0f,0.0f,0.0f);
    fResources.fnvEyePoint = new SphericalPoint();
    fResources.fnvEyePoint.fRho = 10000.0f;
    fRenderer = new Renderer(fWorldX1,fWorldY1,fWorldX2,fWorldY2,
                             0,0,0,0,null,0);
    fLastAction = kNone;

    // adjust the windowlayout
    fAboutBoxShown = false;
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setSize(fWinXSize,fWinYSize);
    setResizable(false);
    setLocation(0,0);
    setTitle(fMessages.lookup("textNetworkViewerWindowTitle"));
    setBackground(Color.black);
    setIconImage(Toolkit.getDefaultToolkit().getImage("M2000Icon.jpg"));

    setLayout(new BorderLayout());

    // create and add the viewer-canvas to the window
    fViewerCanvas = new ViewerCanvas(fWinXSize,fWinYSize);
    add("North",fViewerCanvas);

    // create panel containing network-viewer-control commands
    Panel ControlPanel = new Panel();
    ControlPanel.setLayout(new GridLayout(2,5));

    Button fButton;
    fButton = new Button(fMessages.lookup("buttonRotateXYLeft"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonRotateZLeft"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonIncreaseDistance"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonSetViewPoint"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonAbout"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonRotateXYRight"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonRotateZRight"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonDecreaseDistance"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonViewOptions"));
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    fButton = new Button(fMessages.lookup("buttonQuit"));
    fButton.setBackground(Color.red);
    fButton.addActionListener(this);
    fButton.addKeyListener(this);
    ControlPanel.add(fButton);

    // add the controlpanel to the window
    add("South",ControlPanel);

    // add the windowlistener
    addWindowListener(this);

    pack();
    show();

    if (fResources.fnvShowAboutBox) {
      // show the about-box
      AboutBox about = new AboutBox(this,"Network Viewer");
    }
    fAboutBoxShown = true;
  }

  private void rotateXYLeft()
  {
    fResources.fnvEyePoint.fPhi -= fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
    if (fResources.fnvEyePoint.fPhi > Math.PI) {
      fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi - (2.0f * Math.PI));
    }
    if (fResources.fnvEyePoint.fPhi < -Math.PI) {
      fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi + (2.0f * Math.PI));
    }
    fViewerCanvas.repaint();
    fLastAction = kRotateXYLeft;
  }

  private void rotateXYRight()
  {
    fResources.fnvEyePoint.fPhi += fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
    if (fResources.fnvEyePoint.fPhi > Math.PI) {
      fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi - (2.0f * Math.PI));
    }
    if (fResources.fnvEyePoint.fPhi < -Math.PI) {
      fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi + (2.0f * Math.PI));
    }
    fViewerCanvas.repaint();
    fLastAction = kRotateXYRight;
  }

  private void rotateZLeft()
  {
    fResources.fnvEyePoint.fTheta -= fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
    if (fResources.fnvEyePoint.fTheta >= (2.0f * Math.PI)) {
      fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta - (2.0f * Math.PI));
    }
    if (fResources.fnvEyePoint.fTheta < 0.0f) {
      fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta + (2.0f * Math.PI));
    }
    fViewerCanvas.repaint();
    fLastAction = kRotateZLeft;
  }

  private void rotateZRight()
  {
    fResources.fnvEyePoint.fTheta += fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
    if (fResources.fnvEyePoint.fTheta >= (2.0f * Math.PI)) {
      fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta - (2.0f * Math.PI));
    }
    if (fResources.fnvEyePoint.fTheta < 0.0f) {
      fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta + (2.0f * Math.PI));
    }
    fViewerCanvas.repaint();
    fLastAction = kRotateZRight;
  }

  private void increaseDistance()
  {
    fResources.fnvEyePoint.fRho += fResources.fnvDistanceStepSize;
    fViewerCanvas.repaint();
    fLastAction = kIncreaseDistance;
  }

  private void decreaseDistance()
  {
    fResources.fnvEyePoint.fRho -= fResources.fnvDistanceStepSize;
    if (fResources.fnvEyePoint.fRho < 0.0f) {
      fResources.fnvEyePoint.fRho = 0.0f;
    }
    fViewerCanvas.repaint();
    fLastAction = kDecreaseDistance;
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    // quit the running program
    System.exit(0);
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonRotateXYLeft"))) {
      rotateXYLeft();
    }

    if (Command.equals(fMessages.lookup("buttonRotateXYRight"))) {
      rotateXYRight();
    }

    if (Command.equals(fMessages.lookup("buttonRotateZLeft"))) {
      rotateZLeft();
    }

    if (Command.equals(fMessages.lookup("buttonRotateZRight"))) {
      rotateZRight();
    }

    if (Command.equals(fMessages.lookup("buttonAbout"))) {
      AboutBox fAbout = new AboutBox(this,"Network Viewer");
    }

    if (Command.equals(fMessages.lookup("buttonIncreaseDistance"))) {
      increaseDistance();
    }

    if (Command.equals(fMessages.lookup("buttonDecreaseDistance"))) {
      decreaseDistance();
    }

    if (Command.equals(fMessages.lookup("buttonSetViewPoint"))) {
      ViewPointBox fViewPointBox = new ViewPointBox(this);
      fViewerCanvas.repaint();
    }

    if (Command.equals(fMessages.lookup("buttonViewOptions"))) {
      ViewOptionsBox fViewOptionsBox = new ViewOptionsBox(this);
      fViewerCanvas.repaint();
    }

    if (Command.equals(fMessages.lookup("buttonQuit"))) {
      windowClosing(null);
    }
  }

  // methods from the KeyListener-interface
  public void keyTyped(KeyEvent e) { }
  public void keyPressed(KeyEvent e)
  {
    int KeyCode = e.getKeyCode();

    if (KeyCode == KeyEvent.VK_LEFT) {
      fResources.fnvViewPoint.fX -= fResources.fnvDistanceStepSize;
      fViewerCanvas.repaint();
    }
    else if (KeyCode == KeyEvent.VK_RIGHT) {
      fResources.fnvViewPoint.fX += fResources.fnvDistanceStepSize;
      fViewerCanvas.repaint();
    }
    else if (KeyCode == KeyEvent.VK_UP) {
      fResources.fnvViewPoint.fY += fResources.fnvDistanceStepSize;
      fViewerCanvas.repaint();
    }
    else if (KeyCode == KeyEvent.VK_DOWN) {
      fResources.fnvViewPoint.fY -= fResources.fnvDistanceStepSize;
      fViewerCanvas.repaint();
    }
    else if (KeyCode == KeyEvent.VK_ADD) {
      decreaseDistance();
    }
    else if (KeyCode == KeyEvent.VK_SUBTRACT) {
      increaseDistance();
    }
    else if (KeyCode == KeyEvent.VK_NUMPAD4) {
      rotateZLeft();
    }
    else if (KeyCode == KeyEvent.VK_NUMPAD6) {
      rotateZRight();
    }
    else if (KeyCode == KeyEvent.VK_NUMPAD2) {
      rotateXYRight();
    }
    else if (KeyCode == KeyEvent.VK_NUMPAD8) {
      rotateXYLeft();
    }
  }
  public void keyReleased(KeyEvent e) { }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */
  // the viewer's canvas
  private class ViewerCanvas extends Canvas implements MouseListener
  {
    // the canvas's window coordinates
    private int x1;
    private int y1;
    private int x2;
    private int y2;

    public ViewerCanvas(int width, int height)
    {
      super();

      setSize(width,height);

      // add the mouselistener
      addMouseListener(this);
    }
    
    private void convertNode(Node node, Point3D point)
    {
      point.set(node.fXPos,node.fYPos,node.fZPos);
    }

    public void paint(Graphics g)
    {
      // prevent drawing of the network when the aboutbox is shown at startup
      if (fAboutBoxShown == false) {
        return;
      }

      // define the boundaries for drawing
      x1 = 5;
      y1 = 5;
      x2 = getSize().width - x1 - 5 - getInsets().right;
      y2 = getSize().height - y1 - 5 - getInsets().bottom;

      // configure the 3D-renderer
      fRenderer.setWVMapping(fWorldX1,fWorldY1,fWorldX2,fWorldY2,
                             x1,y1,x2,y2);
      fRenderer.setYOffset(fWinYSize);
      fRenderer.setViewPoint(fResources.fnvViewPoint.fX,
                             fResources.fnvViewPoint.fY,
                             fResources.fnvViewPoint.fZ);
      fRenderer.setEyePoint(fResources.fnvEyePoint.fRho,
                            fResources.fnvEyePoint.fPhi,
                            fResources.fnvEyePoint.fTheta);
      fRenderer.setGraphics(g);
      if (fResources.fnvPerspectiveMode) {
        fRenderer.setPerspectiveMode();
      }
      else {
        fRenderer.setParallelMode();
      }

      // draw a border
      g.setColor(fResources.knvBorderColor);
      g.drawRoundRect(x1,y1,x2,y2,10,10);
      // activate clipping of graphics primitives
      g.setClip(x1 + 1,y1 + 1,x2 - 1,y2 - 1);

      // render the grid
      if (fResources.fnvShowGrid &&
         (fNetwork.fBoundingBox.fUndefined == false)) {

        g.setColor(fResources.knvGridColor);
        Point3D GridPoint = new Point3D();
        GridPoint.fX = fNetwork.fBoundingBox.fMinX;
        GridPoint.fZ = fResources.fnvGridHeight;

        if (fResources.fnvPointGridType) {
          while (GridPoint.fX <= fNetwork.fBoundingBox.fMaxX) {
            GridPoint.fY = fNetwork.fBoundingBox.fMinY;
            while (GridPoint.fY <= fNetwork.fBoundingBox.fMaxY) {
              fRenderer.plotPoint(GridPoint);
              GridPoint.fY += fResources.fnvGridFrequency;
            }
            GridPoint.fX += fResources.fnvGridFrequency;
          }
        }
        else {
          while (GridPoint.fX <= fNetwork.fBoundingBox.fMaxX) {
            Point3D Point1 = new Point3D();
            Point3D Point2 = new Point3D();

            Point1.fX = GridPoint.fX;
            Point1.fY = fNetwork.fBoundingBox.fMinY;
            Point1.fZ = GridPoint.fZ;

            Point2.fX = GridPoint.fX;
            Point2.fY = fNetwork.fBoundingBox.fMaxY;
            Point2.fZ = GridPoint.fZ;

            fRenderer.plotLine(Point1,Point2);
            GridPoint.fX += fResources.fnvGridFrequency;
          }

          GridPoint.fY = fNetwork.fBoundingBox.fMinY;
          while (GridPoint.fY <= fNetwork.fBoundingBox.fMaxY) {
            Point3D Point1 = new Point3D();
            Point3D Point2 = new Point3D();

            Point1.fX = fNetwork.fBoundingBox.fMinX;
            Point1.fY = GridPoint.fY;
            Point1.fZ = GridPoint.fZ;

            Point2.fX = fNetwork.fBoundingBox.fMaxX;
            Point2.fY = GridPoint.fY;
            Point2.fZ = GridPoint.fZ;

            fRenderer.plotLine(Point1,Point2);
            GridPoint.fY += fResources.fnvGridFrequency;
          }
        }
      }

      // handle the network's environment
      if (fResources.fnvShowEnvironment) {

        Enumeration EnvironmentEnum = fNetwork.fEnvironment.elements();
        while (EnvironmentEnum.hasMoreElements()) {
          Environment environment = (Environment) EnvironmentEnum.nextElement();

          g.setColor(environment.fColor);

          if (environment.fType == Environment.kRectangle) {
            Point3D Point[] = new Point3D[4];
            for (int i = 0; i < 4; ++i) {
              Point[i] = new Point3D();
            }

            Point[0].fX = environment.fNodes[0].fXPos;
            Point[0].fY = environment.fNodes[0].fYPos;
            Point[0].fZ = environment.fNodes[0].fZPos;

            Point[1].fX = environment.fNodes[0].fXPos;
            Point[1].fY = environment.fNodes[1].fYPos;
            Point[1].fZ = environment.fNodes[0].fZPos;

            Point[2].fX = environment.fNodes[1].fXPos;
            Point[2].fY = environment.fNodes[1].fYPos;
            Point[2].fZ = environment.fNodes[0].fZPos;

            Point[3].fX = environment.fNodes[1].fXPos;
            Point[3].fY = environment.fNodes[0].fYPos;
            Point[3].fZ = environment.fNodes[0].fZPos;

            if (environment.fFilled) {
              Polygon Rectangle = new Polygon();
              for (int i = 0; i < 4; ++i) {
                ViewportPoint Pos = fRenderer.renderPoint(Point[i]);
                Rectangle.addPoint(Pos.fX,Pos.fY);
              }
              g.fillPolygon(Rectangle);
            }
            else {
              // draw the outline
              fRenderer.plotLine(Point[0],Point[1]);
              fRenderer.plotLine(Point[1],Point[2]);
              fRenderer.plotLine(Point[2],Point[3]);
              fRenderer.plotLine(Point[3],Point[0]);
            }
          }
          else if (environment.fType == Environment.kPolygon) {
            Point3D Point = new Point3D();
            Polygon polygon = new Polygon();
            ViewportPoint Pos = new ViewportPoint();
            for (int i = 0; i < environment.fNrOfNodes; ++i) {
              convertNode(environment.fNodes[i],Point);
              Pos = fRenderer.renderPoint(Point);
              polygon.addPoint(Pos.fX,Pos.fY);
            }

            if (environment.fFilled) {
              g.fillPolygon(polygon);
            }
            else {
              // draw the outline
              g.drawPolygon(polygon);
            }
          }
          else if (environment.fType == Environment.kOval) {
            Point3D Point = new Point3D();
            Polygon polygon = new Polygon();
            ViewportPoint Pos = new ViewportPoint();

            for (int i = 0; i < fResources.fnvOvalInterpolation; ++i) {
              float Angle = (float) (((2.0 * Math.PI) / fResources.fnvOvalInterpolation) * i);
              Point.fX = environment.fNodes[0].fXPos + Math.round((float) environment.fXRadius * Math.cos(Angle));
              Point.fY = environment.fNodes[0].fYPos + Math.round((float) environment.fYRadius * Math.sin(Angle));
              Point.fZ = environment.fNodes[0].fZPos;
              Pos = fRenderer.renderPoint(Point);
              polygon.addPoint(Pos.fX,Pos.fY);
            }

            if (environment.fFilled) {
              g.fillPolygon(polygon);
            }
            else {
              // draw the outline
              g.drawPolygon(polygon);
            }
          }
          else if (environment.fType == Environment.kRailroad) {

            g.setColor(Color.lightGray);

            if (fResources.fnvRenderSplines) {
              Point3D[] Q = new Point3D[4];
              for (int i = 0; i < 4; ++i) {
                Q[i] = new Point3D();
              }

              // draw the first spline-segment
              convertNode(environment.fNodes[0],Q[0]);
              // duplicate first node
              convertNode(environment.fNodes[0],Q[1]);
              convertNode(environment.fNodes[1],Q[2]);
              int LastSegmentNode = 2;
              if (environment.fNrOfNodes == 2) {
                // duplicate the last node
                LastSegmentNode = 1;
              }
              convertNode(environment.fNodes[LastSegmentNode],Q[3]);
              fRenderer.plotSpline(Q[0],Q[1],Q[2],Q[3],fResources.fnvSplineInterpolation,Renderer.kRailroad);

              // draw the intermediate spline-segments
              if (environment.fNrOfNodes >= 4) {
                for (int i = 0; i < (environment.fNrOfNodes - 3); ++i) {
                  convertNode(environment.fNodes[i],Q[0]);
                  convertNode(environment.fNodes[i + 1],Q[1]);
                  convertNode(environment.fNodes[i + 2],Q[2]);
                  convertNode(environment.fNodes[i + 3],Q[3]);
                  fRenderer.plotSpline(Q[0],Q[1],Q[2],Q[3],fResources.fnvSplineInterpolation,Renderer.kRailroad);
                }
              }

              // draw the last spline-segment
              int FirstSegmentNode = environment.fNrOfNodes - 3;
              if (environment.fNrOfNodes == 2) {
                // duplicate the first node
                FirstSegmentNode = 0;
              }   
              convertNode(environment.fNodes[FirstSegmentNode],Q[0]);
              convertNode(environment.fNodes[environment.fNrOfNodes - 2],Q[1]);
              convertNode(environment.fNodes[environment.fNrOfNodes - 1],Q[2]);
              // duplicate last node
              convertNode(environment.fNodes[environment.fNrOfNodes - 1],Q[3]);
              fRenderer.plotSpline(Q[0],Q[1],Q[2],Q[3],fResources.fnvSplineInterpolation,Renderer.kRailroad);
            }
            else {
              Point3D p1 = new Point3D();
              Point3D p2 = new Point3D();
              // draw straight-lines between the linksegments
              for (int i = 0; i < (environment.fNrOfNodes - 1); ++i) {

                convertNode(environment.fNodes[i],p1);
                convertNode(environment.fNodes[i + 1],p2);

                fRenderer.plotRailroad(p1,p2);
              }
            }
          }

          if (fResources.fnvShowIDs) {
            ViewportPoint Pos;
            Point3D Point = new Point3D();

            if (environment.fType == Environment.kRectangle) {
              Point.fX = (environment.fNodes[0].fXPos + environment.fNodes[1].fXPos) / 2;
              Point.fY = (environment.fNodes[0].fYPos + environment.fNodes[1].fYPos) / 2;
            }
            else if (environment.fType == Environment.kPolygon) {
              float SumX = 0.0f;
              float SumY = 0.0f;
              for (int i = 0; i < environment.fNrOfNodes; ++i) {
                SumX += environment.fNodes[i].fXPos;
                SumY += environment.fNodes[i].fYPos;
              }
              Point.fX = SumX / environment.fNrOfNodes;
              Point.fY = SumY / environment.fNrOfNodes;
            }
            else if (environment.fType == Environment.kOval) {
              Point.fX = environment.fNodes[0].fXPos;
              Point.fY = environment.fNodes[0].fYPos;
            }
            Point.fZ = environment.fNodes[0].fZPos;

            Pos = fRenderer.renderPoint(Point);
            g.setColor(fResources.knvIDColor);
            g.drawString(environment.fName,Pos.fX + 10,Pos.fY + 10);
          }
        }
      }

      int XMin = 0;
      int YMin = 0;
      int ZMin = 0;
      int XMax = 0;
      int YMax = 0;
      int ZMax = 0;

      // handle the network's bounding-box
      if (fNetwork.fBoundingBox.fUndefined == false) {

        // calculate the bounding-box for the network
        XMin = fNetwork.fBoundingBox.fMinX;
        YMin = fNetwork.fBoundingBox.fMinY;
        ZMin = fNetwork.fBoundingBox.fMinZ;
        XMax = fNetwork.fBoundingBox.fMaxX;
        YMax = fNetwork.fBoundingBox.fMaxY;
        ZMax = fNetwork.fBoundingBox.fMaxZ;

        if (fResources.fnvShowBoundingBox) {

          // show the bounding box
          Point3D[] boundingBox = new Point3D[8];
          boundingBox[0] = new Point3D(XMax,YMin,ZMin);
          boundingBox[1] = new Point3D(XMax,YMax,ZMin);
          boundingBox[2] = new Point3D(XMin,YMax,ZMin);
          boundingBox[3] = new Point3D(XMin,YMin,ZMin);
          boundingBox[4] = new Point3D(XMax,YMin,ZMax);
          boundingBox[5] = new Point3D(XMax,YMax,ZMax);
          boundingBox[6] = new Point3D(XMin,YMax,ZMax);
          boundingBox[7] = new Point3D(XMin,YMin,ZMax);

          g.setColor(fResources.knvBoundingBoxColor);

          fRenderer.plotLine(boundingBox[0],boundingBox[1]);
          fRenderer.plotLine(boundingBox[1],boundingBox[2]);
          fRenderer.plotLine(boundingBox[2],boundingBox[3]);
          fRenderer.plotLine(boundingBox[3],boundingBox[0]);

          fRenderer.plotLine(boundingBox[4],boundingBox[5]);
          fRenderer.plotLine(boundingBox[5],boundingBox[6]);
          fRenderer.plotLine(boundingBox[6],boundingBox[7]);
          fRenderer.plotLine(boundingBox[7],boundingBox[4]);

          fRenderer.plotLine(boundingBox[0],boundingBox[4]);
          fRenderer.plotLine(boundingBox[1],boundingBox[5]);
          fRenderer.plotLine(boundingBox[2],boundingBox[6]);
          fRenderer.plotLine(boundingBox[3],boundingBox[7]);

          if (fResources.fnvShowIDs) {
            ViewportPoint Pos;
            Pos = fRenderer.renderPoint(boundingBox[3]);
            g.drawString("(" +
                         String.valueOf(Math.round(boundingBox[3].fX)) + "," +
                         String.valueOf(Math.round(boundingBox[3].fY)) + "," +
                         String.valueOf(Math.round(boundingBox[3].fZ)) + ")",
                         Pos.fX + 10,Pos.fY + 10);

            Pos = fRenderer.renderPoint(boundingBox[5]);
            g.drawString("(" +
                         String.valueOf(Math.round(boundingBox[5].fX)) + "," +
                         String.valueOf(Math.round(boundingBox[5].fY)) + "," +
                         String.valueOf(Math.round(boundingBox[5].fZ)) + ")",
                         Pos.fX + 10,Pos.fY + 10);
          }
        }
      }
      else {
        // assume default values so the axes can still be drawn
        XMax = 1;
        YMax = 1;
        ZMax = 1;
      }

      // handle the 3D-axes
      if (fResources.fnvShowAxes) {
        Point3D[] Axes = new Point3D[12];

        // find the longest axis and show all axes with equal length
        float AxisLength = Math.abs(XMax - XMin);
        if (Math.abs(YMax - YMin) > AxisLength) {
          AxisLength = Math.abs(YMax - YMin);
        }
        if (Math.abs(ZMax - ZMin) > AxisLength) {
          AxisLength = Math.abs(ZMax - ZMin);
        }

        Axes[0] = new Point3D(-(AxisLength / 20.0f),0.0f,0.0f);
        Axes[1] = new Point3D((AxisLength / 2.0f),0.0f,0.0f);
        Axes[2] = new Point3D((AxisLength / 2.0f) - (AxisLength / 20.0f),
                              -(AxisLength / 20.0f),0.0f);
        Axes[3] = new Point3D((AxisLength / 2.0f) - (AxisLength / 20.0f),
                              (AxisLength / 20.0f),0.0f);

        Axes[4] = new Point3D(0.0f,-(AxisLength / 20.0f),0.0f);
        Axes[5] = new Point3D(0.0f,(AxisLength / 2.0f),0.0f);
        Axes[6] = new Point3D(-(AxisLength / 20.0f),
                              (AxisLength / 2.0f) - (AxisLength / 20.0f),0.0f);
        Axes[7] = new Point3D((AxisLength / 20.0f),
                              (AxisLength / 2.0f) - (AxisLength / 20.0f),0.0f);

        Axes[8] = new Point3D(0.0f,0.0f,-(AxisLength / 20.0f));
        Axes[9] = new Point3D(0.0f,0.0f,(AxisLength / 2.0f));
        Axes[10] = new Point3D(-(AxisLength / 20.0f),
                               (AxisLength / 20.0f),
                               (AxisLength / 2.0f) - (AxisLength / 20.0f));
        Axes[11] = new Point3D((AxisLength / 20.0f),
                               -(AxisLength / 20.0f),
                               (AxisLength / 2.0f) - (AxisLength / 20.0f));

        g.setColor(fResources.knvAxesColor);
        fRenderer.plotLine(Axes[0],Axes[1]);
        fRenderer.plotLine(Axes[1],Axes[2]);
        fRenderer.plotLine(Axes[1],Axes[3]);

        fRenderer.plotLine(Axes[4],Axes[5]);
        fRenderer.plotLine(Axes[5],Axes[6]);
        fRenderer.plotLine(Axes[5],Axes[7]);

        fRenderer.plotLine(Axes[8],Axes[9]);
        fRenderer.plotLine(Axes[9],Axes[10]);
        fRenderer.plotLine(Axes[9],Axes[11]);

        ViewportPoint Pos = fRenderer.renderPoint(Axes[0]);
        g.drawString("0",Pos.fX + 10,Pos.fY + 10);

        Pos = fRenderer.renderPoint(Axes[1]);
        g.drawString("X",Pos.fX + 10,Pos.fY + 10);

        Pos = fRenderer.renderPoint(Axes[5]);
        g.drawString("Y",Pos.fX + 10,Pos.fY + 10);

        Pos = fRenderer.renderPoint(Axes[9]);
        g.drawString("Z",Pos.fX + 10,Pos.fY + 10);
      }

      // handle the network's links
      if (fNetwork.fLinks.size() != 0) {

        if (fResources.fnvShowLinks) {

          // show the links
          Enumeration LinkEnum = fNetwork.fLinks.elements();
          while (LinkEnum.hasMoreElements()) {
            Link link = (Link) LinkEnum.nextElement();

            g.setColor(fResources.knvLinkColor);

            // draw each segment in the link
            if (fResources.fnvRenderSplines) {

              Point3D[] Q = new Point3D[4];
              for (int i = 0; i < 4; ++i) {
                Q[i] = new Point3D();
              }

              // draw the first spline-segment
              if (link.fMainLinksPreviousLink != null) {
                convertNode(link.fMainLinksPreviousLink.fNodes[link.fMainLinksPreviousLink.fNrOfNodes - 2],Q[0]);
              }
              else if (link.fOnRampsPreviousLink != null) {
                convertNode(link.fOnRampsPreviousLink.fNodes[link.fOnRampsPreviousLink.fNrOfNodes - 2],Q[0]);
              }
              else {
                // duplicate first node
                convertNode(link.fNodes[0],Q[0]);
              }
              convertNode(link.fNodes[0],Q[1]);
              convertNode(link.fNodes[1],Q[2]);

              if (link.fNrOfNodes > 2) {
                convertNode(link.fNodes[2],Q[3]);
              }
              else {
                if (link.fMainLinksNextLink != null) {
                  convertNode(link.fMainLinksNextLink.fNodes[1],Q[3]);
                }
                else if (link.fOffRampsNextLink != null) {
                  convertNode(link.fOffRampsNextLink.fNodes[1],Q[3]);
                }
                else {
                  // duplicate last node
                  convertNode(link.fNodes[1],Q[3]);
                }
              }
              fRenderer.plotSpline(Q[0],Q[1],Q[2],Q[3],fResources.fnvSplineInterpolation,Renderer.kLine);

              // draw the intermediate spline-segments
              if (link.fNrOfNodes >= 4) {
                for (int i = 0; i < (link.fNrOfNodes - 3); ++i) {
                  convertNode(link.fNodes[i],Q[0]);
                  convertNode(link.fNodes[i + 1],Q[1]);
                  convertNode(link.fNodes[i + 2],Q[2]);
                  convertNode(link.fNodes[i + 3],Q[3]);
                  fRenderer.plotSpline(Q[0],Q[1],Q[2],Q[3],fResources.fnvSplineInterpolation,Renderer.kLine);
                }
              }

              // draw the last spline-segment
              if (link.fNrOfNodes > 2) {
                convertNode(link.fNodes[link.fNrOfNodes - 3],Q[0]);
              }
              else {
                if (link.fMainLinksPreviousLink != null) {
                  convertNode(link.fMainLinksPreviousLink.fNodes[link.fMainLinksPreviousLink.fNrOfNodes - 2],Q[0]);
                }
                else if (link.fOnRampsPreviousLink != null) {
                  convertNode(link.fOnRampsPreviousLink.fNodes[link.fOnRampsPreviousLink.fNrOfNodes - 2],Q[0]);
                }
                else {
                  // duplicate first node
                  convertNode(link.fNodes[0],Q[0]);
                }
              }
              convertNode(link.fNodes[link.fNrOfNodes - 2],Q[1]);
              convertNode(link.fNodes[link.fNrOfNodes - 1],Q[2]);
              if (link.fMainLinksNextLink != null) {
                convertNode(link.fMainLinksNextLink.fNodes[1],Q[3]);
              }
              else if (link.fOffRampsNextLink != null) {
                convertNode(link.fOffRampsNextLink.fNodes[1],Q[3]);
              }
              else {
                // duplicate last node
                convertNode(link.fNodes[link.fNrOfNodes - 1],Q[3]);
              }
              fRenderer.plotSpline(Q[0],Q[1],Q[2],Q[3],fResources.fnvSplineInterpolation,Renderer.kLine);
            }
            else {
              Point3D p1 = new Point3D();
              Point3D p2 = new Point3D();
              g.setColor(fResources.knvLinkColor);
              // draw straight-lines between the linksegments
              for (int i = 0; i < (link.fNrOfNodes - 1); ++i) {

                convertNode(link.fNodes[i],p1);
                convertNode(link.fNodes[i + 1],p2);

                fRenderer.plotLine(p1,p2);
              }
            }
          }

          if (fResources.fnvShowIDs) {
            g.setColor(fResources.knvIDColor);

            LinkEnum = fNetwork.fLinks.elements();
            while (LinkEnum.hasMoreElements()) {
              Link link = (Link) LinkEnum.nextElement();

              Point3D p1 = new Point3D();
              Point3D p2 = new Point3D();

              for (int i = 0; i < (link.fNrOfNodes - 1); ++i) {

                convertNode(link.fNodes[i],p1);
                convertNode(link.fNodes[i + 1],p2);
             
                Point3D MidwayCoords = new Point3D();
                MidwayCoords.fX = (p1.fX + p2.fX) / 2;
                MidwayCoords.fY = (p1.fY + p2.fY) / 2;
                MidwayCoords.fZ = (p1.fZ + p2.fZ) / 2;
                ViewportPoint Pos = fRenderer.renderPoint(MidwayCoords);
                g.drawString("l",Pos.fX,Pos.fY);
                if (link.fName.length() != 0) {
                  g.drawString(String.valueOf(link.fID) + " (" + link.fName + ")",
                               Pos.fX + 5,Pos.fY + 8);
                }
                else {
                  g.drawString(String.valueOf(link.fID),Pos.fX + 5,Pos.fY + 8);
                }
              }
            }
          }        
        }
      }

      // handle the network's nodes
      if (fNetwork.fNodes.size() != 0) {

        if (fResources.fnvShowNodes) {

          // show the nodes
          Enumeration NodeEnum = fNetwork.fNodes.elements();
          while (NodeEnum.hasMoreElements()) {
            Node node = (Node) NodeEnum.nextElement();
            Point3D NodeCoords = new Point3D();
            NodeCoords.fX = node.fXPos;
            NodeCoords.fY = node.fYPos;
            NodeCoords.fZ = node.fZPos;
            g.setColor(fResources.knvNodeColor);
            fRenderer.plotDot(NodeCoords);

            if (fResources.fnvShowIDs) {
              ViewportPoint Pos = fRenderer.renderPoint(NodeCoords);
              g.setColor(fResources.knvIDColor);
              g.drawString("n",Pos.fX + 10,Pos.fY + 10);
              if (node.fName.length() != 0) {
                g.drawString(String.valueOf(node.fID) +
                             " (" + node.fName + ")",Pos.fX + 18,Pos.fY + 18);
              }
              else {
                g.drawString(String.valueOf(node.fID),Pos.fX + 18,Pos.fY + 18);
              }
            }
          }
        }
      }

      // show general information
      g.setColor(fResources.knvInformationColor);

      String Perspective = fResources.fMessages.lookup("textPerspectiveModeInformation");
      if (fResources.fnvPerspectiveMode == false) {
        Perspective = fResources.fMessages.lookup("textParallelModeInformation");
      }

      g.drawString(fMessages.lookup("textNetwork") + " : " + fNetwork.fTitle +
                   " (#" + fMessages.lookup("textNodes") + " = " + String.valueOf(fNetwork.fNodes.size()) +
                           ", #" + fMessages.lookup("textLinks") + " = " + String.valueOf(fNetwork.fLinks.size()) + ")" +
                   "    " + Perspective,
                   25,35);

      Point3D EyeCoords = fRenderer.convertSphericalToCartesian(fResources.fnvEyePoint);
      g.drawString(fMessages.lookup("textEyeCoords") + " = (" +
                   String.valueOf(Math.round(EyeCoords.fX)) + "," +
                   String.valueOf(Math.round(EyeCoords.fY)) + "," +
                   String.valueOf(Math.round(EyeCoords.fZ)) + ") ~ (" +
                   String.valueOf(Math.round(fResources.fnvEyePoint.fRho)) + "," +
                   String.valueOf(Math.round(fRenderer.convertRadToDeg(fResources.fnvEyePoint.fPhi))) + " ," +
                   String.valueOf(Math.round(fRenderer.convertRadToDeg(fResources.fnvEyePoint.fTheta))) + " )",
                   25,55);

      Point3D ViewPointCoords = fResources.fnvViewPoint;
      g.drawString(fMessages.lookup("textViewPointCoords") + " = (" +
                   String.valueOf(Math.round(ViewPointCoords.fX)) + "," +
                   String.valueOf(Math.round(ViewPointCoords.fY)) + "," +
                   String.valueOf(Math.round(ViewPointCoords.fZ)) + ")",
                   25,75);
    }

    // methods from the MouseListener-interface
    public void mouseClicked(MouseEvent e)
    {
      // was the mousebutton doubleclicked ?
      if (e.getClickCount() == 2) {
        int x = e.getX();
        int y = e.getY();

        if (fLastAction == kRotateXYLeft) {
          for (int i = 0; i < fResources.fnvUpdateSteps; ++i) {
            fResources.fnvEyePoint.fPhi -= fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
            if (fResources.fnvEyePoint.fPhi > Math.PI) {
              fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi - (2.0f * Math.PI));
            }
            if (fResources.fnvEyePoint.fPhi < -Math.PI) {
              fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi + (2.0f * Math.PI));
            }
            update(getGraphics());
            try {
              Thread.sleep(fResources.fnvUpdateDelayMs);
            }
            catch (Exception exc) {
            }
          }
        }
        else if (fLastAction == kRotateXYRight) {
          for (int i = 0; i < fResources.fnvUpdateSteps; ++i) {
            fResources.fnvEyePoint.fPhi += fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
            if (fResources.fnvEyePoint.fPhi > Math.PI) {
              fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi - (2.0f * Math.PI));
            }
            if (fResources.fnvEyePoint.fPhi < -Math.PI) {
              fResources.fnvEyePoint.fPhi = (float) (fResources.fnvEyePoint.fPhi + (2.0f * Math.PI));
            }
            update(getGraphics());
            try {
              Thread.sleep(fResources.fnvUpdateDelayMs);
            }
            catch (Exception exc) {
            }
          }
        }
        else if (fLastAction == kRotateZLeft) {
          for (int i = 0; i < fResources.fnvUpdateSteps; ++i) {
            fResources.fnvEyePoint.fTheta -= fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
            if (fResources.fnvEyePoint.fTheta >= (2.0f * Math.PI)) {
              fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta - (2.0f * Math.PI));
            }
            if (fResources.fnvEyePoint.fTheta < 0.0f) {
              fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta + (2.0f * Math.PI));
            }
            update(getGraphics());
            try {
              Thread.sleep(fResources.fnvUpdateDelayMs);
            }
            catch (Exception exc) {
            }
          }
        }
        else if (fLastAction == kRotateZRight) {
          for (int i = 0; i < fResources.fnvUpdateSteps; ++i) {
            fResources.fnvEyePoint.fTheta += fRenderer.convertDegToRad(fResources.fnvRotateStepSize);
            if (fResources.fnvEyePoint.fTheta >= (2.0f * Math.PI)) {
              fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta - (2.0f * Math.PI));
            }
            if (fResources.fnvEyePoint.fTheta < 0.0f) {
              fResources.fnvEyePoint.fTheta = (float) (fResources.fnvEyePoint.fTheta + (2.0f * Math.PI));
            }
            update(getGraphics());
            try {
              Thread.sleep(fResources.fnvUpdateDelayMs);
            }
            catch (Exception exc) {
            }
          }
        }
        else if (fLastAction == kIncreaseDistance) {
          for (int i = 0; i < fResources.fnvUpdateSteps; ++i) {
            fResources.fnvEyePoint.fRho += fResources.fnvDistanceStepSize;
            if (fResources.fnvEyePoint.fRho < 0.0f) {
              fResources.fnvEyePoint.fRho = 0.0f;
            }
            update(getGraphics());
            try {
              Thread.sleep(fResources.fnvUpdateDelayMs);
            }
            catch (Exception exc) {
            }
          }
        }
        else if (fLastAction == kDecreaseDistance) {
          for (int i = 0; i < fResources.fnvUpdateSteps; ++i) {
            fResources.fnvEyePoint.fRho -= fResources.fnvDistanceStepSize;
            if (fResources.fnvEyePoint.fRho < 0.0f) {
              fResources.fnvEyePoint.fRho = 0.0f;
            }
            update(getGraphics());
            try {
              Thread.sleep(fResources.fnvUpdateDelayMs);
            }
            catch (Exception exc) {
            }
          }
        }
      }
    }
    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
  }
}
